"""
Star positioning algorithms for iztro-py

Contains core algorithms for calculating positions of major stars,
especially Ziwei (紫微) and Tianfu (天府).
"""

from typing import Dict, Tuple
from datetime import date, timedelta
from iztro_py.data.types import FiveElementsClass, HeavenlyStemName, EarthlyBranchName
from iztro_py.data.constants import fix_index, ZIWEI_START_POSITIONS
from iztro_py.utils.calendar import parse_solar_date, solar_to_lunar


def get_ziwei_index(five_elements_class: FiveElementsClass, lunar_day: int) -> int:
    """
    计算紫微星所在宫位索引

    口诀：六五四三二，酉午亥辰丑

    算法：
    1. 根据五行局确定起始宫位
       - 水二局：从酉宫(9)起
       - 木三局：从午宫(6)起
       - 金四局：从亥宫(11)起
       - 土五局：从辰宫(4)起
       - 火六局：从丑宫(1)起

    2. 从起始宫位开始，按照五行局数值为一个循环
       每个循环对应一定天数的农历日期

    3. 循环计算直到找到包含农历日的范围

    Args:
        five_elements_class: 五行局
        lunar_day: 农历日 (1-30)

    Returns:
        紫微星所在宫位索引 (0-11)
    """
    # 五行局数值
    class_value = five_elements_class.value  # 2, 3, 4, 5, or 6

    # 起始宫位索引
    # 水2->酉9, 木3->午6, 金4->亥11, 土5->辰4, 火6->丑1
    start_positions = {
        2: 9,  # 水二局：酉宫
        3: 6,  # 木三局：午宫
        4: 11,  # 金四局：亥宫
        5: 4,  # 土五局：辰宫
        6: 1,  # 火六局：丑宫
    }

    start_pos = start_positions[class_value]

    # 计算紫微星位置
    # 算法：(农历日期 - 1) // 五行局数值 = 循环次数
    # 每个循环移动一个宫位
    cycles = (lunar_day - 1) // class_value
    # 与原版 iztro 不完全一致的简化算法，已被 get_start_indices 替代
    ziwei_index = fix_index(start_pos + cycles)

    return ziwei_index


def get_tianfu_index(ziwei_index: int) -> int:
    """
    根据紫微星位置计算天府星位置

    天府星与紫微星的关系：
    紫微在寅，天府在申（相对）
    规律：天府索引 = 对宫 = (紫微索引 + 6) % 12

    实际上天府的计算是：
    寅宫(紫微) -> 申宫(天府)
    如果紫微在寅(2)，天府在子(0): 12 - 2 = 10, 不对

    正确规律：
    紫微寅(2) -> 天府子(0)
    紫微卯(3) -> 天府亥(11)
    紫微辰(4) -> 天府戌(10)
    ...

    实际公式：天府索引 = (12 - 紫微索引) % 12

    Args:
        ziwei_index: 紫微星宫位索引

    Returns:
        天府星宫位索引
    """
    # 修正后的公式
    tianfu_index = fix_index(12 - ziwei_index)

    return tianfu_index


# Backward-compat helper to satisfy existing tests/imports
def get_star_indices(five_elements_class: FiveElementsClass, lunar_day: int) -> Tuple[int, int]:
    """
    兼容旧接口：根据五行局与农历日返回紫微与天府索引

    注意：该算法为简化版，生产使用请改用 get_start_indices。
    """
    ziwei_index = get_ziwei_index(five_elements_class, lunar_day)
    tianfu_index = get_tianfu_index(ziwei_index)
    return ziwei_index, tianfu_index


def get_start_indices(
    solar_date_str: str,
    time_index: int,
    fix_leap: bool,
    heavenly_stem_of_soul: HeavenlyStemName,
    earthly_branch_of_soul: EarthlyBranchName,
) -> Tuple[int, int]:
    """
    计算紫微与天府起始索引（与原生 iztro 对齐）

    对应 iztro/lib/star/location.getStartIndex 的 Python 实现。

    规则要点：
    - “六五四三二，酉午亥辰丑；局数除日数，商数宫前走；若见数无余，便要起虎口，日数小于局，还直宫中守。”
    - 晚子时（time_index==12）按次日处理（跨月则顺延到下一月初一）
    - 五行局按命宫干支起局

    Returns:
        (紫微索引, 天府索引)
    """
    # 解析阳历日期并获取当日农历
    year, month, day = parse_solar_date(solar_date_str)

    # 晚子时按次日计算农历日
    if time_index == 12:
        d = date(year, month, day) + timedelta(days=1)
        lunar = solar_to_lunar(d.year, d.month, d.day, fix_leap)
        lunar_day = lunar.day
    else:
        lunar = solar_to_lunar(year, month, day, fix_leap)
        lunar_day = lunar.day

    # 五行局数值
    # 直接使用传入的命宫干支计算的五行局数值
    # 复用已有查表逻辑
    from iztro_py.utils.helpers import get_five_elements_class

    five_cls = get_five_elements_class(heavenly_stem_of_soul, earthly_branch_of_soul)
    class_value: int = five_cls.value

    # 寻找最小 offset 使 (lunar_day + offset) 能被 class_value 整除
    offset = -1
    remainder = -1
    while remainder != 0:
        offset += 1
        divisor = lunar_day + offset
        remainder = divisor % class_value
        quotient = divisor // class_value

    # 商对 12 取模（以寅为0的坐标系）
    quotient %= 12

    # 起始紫微索引（寅为0坐标系）
    node_ziwei_index = quotient - 1

    # 循环次数为偶数：逆时针加 offset；奇数：顺时针减 offset
    if offset % 2 == 0:
        node_ziwei_index += offset
    else:
        node_ziwei_index -= offset
    node_ziwei_index = fix_index(node_ziwei_index)

    # 天府星位置与紫微星相对（同一坐标系下）
    node_tianfu_index = fix_index(12 - node_ziwei_index)

    # iztro 的索引以寅宫为0，而本库以子宫为0，需要转换：ours = (iztro + 2) % 12
    ziwei_index = fix_index(node_ziwei_index + 2)
    tianfu_index = fix_index(node_tianfu_index + 2)
    return ziwei_index, tianfu_index


def get_major_star_positions(ziwei_index: int, tianfu_index: int) -> Dict[str, int]:
    """
    根据紫微和天府星位置，计算其他主星位置

    紫微星系（逆行）：紫微、天机、太阳、武曲、天同、廉贞
    天府星系（顺行）：天府、太阴、贪狼、巨门、天相、天梁、七杀、破军

    Args:
        ziwei_index: 紫微星宫位索引
        tianfu_index: 天府星宫位索引

    Returns:
        星曜名称到宫位索引的映射字典
    """
    from iztro_py.data.constants import ZIWEI_GROUP, TIANFU_GROUP

    positions = {}

    # 紫微星系（逆行）
    # 紫微、天机、(空)、太阳、武曲、天同、(空)、(空)、廉贞
    for offset, star_name in enumerate(ZIWEI_GROUP):
        if star_name:  # 跳过空位
            # 逆行：索引递减
            star_index = fix_index(ziwei_index - offset)
            positions[star_name] = star_index

    # 天府星系（顺行）
    # 天府、太阴、贪狼、巨门、天相、天梁、七杀、(空)、(空)、(空)、破军
    for offset, star_name in enumerate(TIANFU_GROUP):
        if star_name:  # 跳过空位
            # 顺行：索引递增
            star_index = fix_index(tianfu_index + offset)
            positions[star_name] = star_index

    return positions


def get_minor_star_position_zuofu(lunar_month: int) -> int:
    """
    计算左辅星位置（按农历月）

    口诀：正月起辰宫，顺行十二宫

    Args:
        lunar_month: 农历月 (1-12)

    Returns:
        左辅星宫位索引
    """
    # 正月在辰宫(4)
    return fix_index(4 + lunar_month - 1)


def get_minor_star_position_youbi(lunar_month: int) -> int:
    """
    计算右弼星位置（按农历月）

    口诀：正月起戌宫，逆行十二宫

    Args:
        lunar_month: 农历月 (1-12)

    Returns:
        右弼星宫位索引
    """
    # 正月在戌宫(10)
    return fix_index(10 - (lunar_month - 1))


def get_minor_star_position_wenchang(time_index: int) -> int:
    """
    计算文昌星位置（按时辰）

    口诀：子时起戌宫，逆行十二宫

    Args:
        time_index: 时辰索引 (0-12)

    Returns:
        文昌星宫位索引
    """
    # 处理早子时和晚子时
    actual_time_index = 0 if time_index == 12 else time_index

    # 子时在戌宫(10)，逆行
    return fix_index(10 - actual_time_index)


def get_minor_star_position_wenqu(time_index: int) -> int:
    """
    计算文曲星位置（按时辰）

    口诀：子时起辰宫，顺行十二宫

    Args:
        time_index: 时辰索引 (0-12)

    Returns:
        文曲星宫位索引
    """
    # 处理早子时和晚子时
    actual_time_index = 0 if time_index == 12 else time_index

    # 子时在辰宫(4)，顺行
    return fix_index(4 + actual_time_index)


def get_minor_star_positions_kuiyue(year_stem_index: int) -> Tuple[int, int]:
    """
    计算天魁天钺星位置（按年干）

    Args:
        year_stem_index: 年干索引 (0-9)

    Returns:
        (天魁星索引, 天钺星索引) 元组
    """
    # 天魁位置（按年干）
    # 甲戊庚在丑寅, 乙己在子申, 丙丁在亥酉, 辛在午寅, 壬在卯巳, 癸在卯巳
    kuai_positions = [1, 0, 11, 11, 1, 0, 6, 6, 3, 3]  # 索引对应甲-癸

    # 天钺位置（按年干）
    yue_positions = [7, 8, 9, 9, 7, 8, 2, 2, 5, 5]  # 索引对应甲-癸

    return kuai_positions[year_stem_index], yue_positions[year_stem_index]


def get_minor_star_positions_huoling(year_branch_index: int, time_index: int) -> Tuple[int, int]:
    """
    计算火星铃星位置（按年支和时辰）

    Args:
        year_branch_index: 年支索引 (0-11)
        time_index: 时辰索引 (0-12)

    Returns:
        (火星索引, 铃星索引) 元组
    """
    # 处理早子时和晚子时
    actual_time_index = 0 if time_index == 12 else time_index

    # 火星位置（寅午戌年起寅，申子辰年起申，巳酉丑年起亥，亥卯未年起巳）
    # 年支分组
    if year_branch_index in [2, 6, 10]:  # 寅午戌
        huo_base = 2  # 寅
    elif year_branch_index in [8, 0, 4]:  # 申子辰
        huo_base = 8  # 申
    elif year_branch_index in [5, 9, 1]:  # 巳酉丑
        huo_base = 11  # 亥
    else:  # 亥卯未 [11, 3, 7]
        huo_base = 5  # 巳

    huo_index = fix_index(huo_base + actual_time_index)

    # 铃星位置（卯酉年起卯，寅午戌年起寅...）
    if year_branch_index in [3, 9]:  # 卯酉
        ling_base = 3  # 卯
    elif year_branch_index in [2, 6, 10]:  # 寅午戌
        ling_base = 2  # 寅
    elif year_branch_index in [8, 0, 4]:  # 申子辰
        ling_base = 8  # 申
    else:  # 巳酉丑、亥卯未
        ling_base = 5  # 巳

    ling_index = fix_index(ling_base + actual_time_index)

    return huo_index, ling_index


def get_minor_star_positions_kongjie(time_index: int) -> Tuple[int, int]:
    """
    计算地空地劫星位置（按时辰）

    Args:
        time_index: 时辰索引 (0-12)

    Returns:
        (地空索引, 地劫索引) 元组
    """
    # 处理早子时和晚子时
    actual_time_index = 0 if time_index == 12 else time_index

    # 地空：子时在亥，逆行
    kong_index = fix_index(11 - actual_time_index)

    # 地劫：子时在亥，顺行
    jie_index = fix_index(11 + actual_time_index)

    return kong_index, jie_index


def get_minor_star_positions_lucun_yangtuo_tianma(
    year_stem_index: int, year_branch_index: int
) -> Tuple[int, int, int, int]:
    """
    计算禄存、擎羊、陀罗、天马星位置（按年干支）

    Args:
        year_stem_index: 年干索引 (0-9)
        year_branch_index: 年支索引 (0-11)

    Returns:
        (禄存索引, 擎羊索引, 陀罗索引, 天马索引) 元组
    """
    # 禄存位置（按年干，对应地支）
    # 甲禄在寅、乙禄在卯、丙戊禄在巳、丁己禄在午、庚禄在申、辛禄在酉、壬禄在亥、癸禄在子
    lucun_positions = [2, 3, 5, 6, 5, 6, 8, 9, 11, 0]

    lucun_index = lucun_positions[year_stem_index]

    # 擎羊：禄存的下一宫
    yang_index = fix_index(lucun_index + 1)

    # 陀罗：禄存的上一宫
    tuo_index = fix_index(lucun_index - 1)

    # 天马位置（按年支）
    # 寅午戌年在申、申子辰年在寅、巳酉丑年在亥、亥卯未年在巳
    if year_branch_index in [2, 6, 10]:  # 寅午戌
        tianma_index = 8  # 申
    elif year_branch_index in [8, 0, 4]:  # 申子辰
        tianma_index = 2  # 寅
    elif year_branch_index in [5, 9, 1]:  # 巳酉丑
        tianma_index = 11  # 亥
    else:  # 亥卯未
        tianma_index = 5  # 巳

    return lucun_index, yang_index, tuo_index, tianma_index



"""
Star brightness (星耀亮度) data and calculations

Contains brightness information for stars based on their palace positions.
"""

from typing import Any, Dict, List, Optional
from iztro_py.data.types import StarName, Brightness, EarthlyBranchName
from iztro_py.data.constants import EARTHLY_BRANCHES


# ============================================================================
# Brightness Data for Major Stars
# ============================================================================
# 亮度顺序：寅卯辰巳午未申酉戌亥子丑
# Brightness levels: 庙(miao)、旺(wang)、得(de)、利(li)、平(ping)、不(bu)、陷(xian)

STAR_BRIGHTNESS: Dict[StarName, List[Optional[Brightness]]] = {
    # 紫微星
    "ziweiMaj": ["旺", "旺", "得", "旺", "庙", "庙", "旺", "旺", "得", "旺", "平", "庙"],
    # 天机星
    "tianjiMaj": ["旺", "旺", "平", "陷", "平", "平", "平", "平", "平", "平", "庙", "庙"],
    # 太阳星
    "taiyangMaj": ["庙", "庙", "庙", "庙", "旺", "利", "平", "陷", "陷", "陷", "陷", "得"],
    # 武曲星
    "wuquMaj": ["平", "陷", "得", "利", "陷", "旺", "旺", "旺", "庙", "庙", "得", "得"],
    # 天同星
    "tiantongMaj": ["得", "陷", "陷", "得", "利", "旺", "得", "得", "利", "旺", "庙", "庙"],
    # 廉贞星
    "lianzhenMaj": ["陷", "陷", "陷", "庙", "旺", "利", "平", "平", "平", "平", "庙", "旺"],
    # 天府星
    "tianfuMaj": ["庙", "庙", "庙", "庙", "庙", "庙", "庙", "庙", "庙", "庙", "庙", "庙"],
    # 太阴星
    "taiyinMaj": ["得", "得", "平", "陷", "旺", "庙", "平", "得", "平", "陷", "庙", "庙"],
    # 贪狼星
    "tanlangMaj": ["旺", "旺", "陷", "陷", "利", "得", "庙", "旺", "庙", "旺", "陷", "得"],
    # 巨门星
    "jumenMaj": ["得", "得", "陷", "陷", "陷", "平", "庙", "旺", "平", "得", "旺", "得"],
    # 天相星
    "tianxiangMaj": ["旺", "旺", "平", "平", "庙", "庙", "旺", "旺", "平", "平", "庙", "庙"],
    # 天梁星
    "tianliangMaj": ["庙", "庙", "平", "平", "旺", "旺", "平", "平", "利", "利", "得", "得"],
    # 七杀星
    "qishaMaj": ["庙", "庙", "平", "平", "陷", "陷", "旺", "旺", "得", "得", "利", "利"],
    # 破军星
    "pojunMaj": ["得", "得", "陷", "旺", "庙", "平", "陷", "平", "旺", "庙", "陷", "得"],
    # 辅星的亮度通常不标注，这里可以留空或使用默认值
}


def get_star_brightness(
    star_name: StarName, palace_branch: EarthlyBranchName
) -> Optional[Brightness]:
    """
    根据星曜名称和宫位地支获取星曜亮度

    Args:
        star_name: 星曜名称
        palace_branch: 宫位地支

    Returns:
        亮度等级，如果没有定义则返回None
    """
    if star_name not in STAR_BRIGHTNESS:
        return None

    brightness_list = STAR_BRIGHTNESS[star_name]

    # 地支索引：子丑寅卯辰巳午未申酉戌亥 (0-11)
    # 亮度列表索引：寅卯辰巳午未申酉戌亥子丑 (0-11)
    # 需要转换索引
    branch_index = EARTHLY_BRANCHES.index(palace_branch)

    # 转换索引：子(0)->10, 丑(1)->11, 寅(2)->0, ...
    brightness_index_map = {
        0: 10,  # 子
        1: 11,  # 丑
        2: 0,  # 寅
        3: 1,  # 卯
        4: 2,  # 辰
        5: 3,  # 巳
        6: 4,  # 午
        7: 5,  # 未
        8: 6,  # 申
        9: 7,  # 酉
        10: 8,  # 戌
        11: 9,  # 亥
    }

    brightness_index = brightness_index_map[branch_index]

    return brightness_list[brightness_index]


def apply_brightness_to_palaces(palaces: List[Dict[str, Any]]) -> None:
    """
    为所有宫位中的星曜添加亮度属性

    Args:
        palaces: 宫位列表

    Note:
        直接修改palaces列表，不返回值
    """
    for palace in palaces:
        palace_branch = palace["earthly_branch"]

        # 为主星添加亮度
        for star in palace["major_stars"]:
            brightness = get_star_brightness(star.name, palace_branch)
            if brightness:
                star.brightness = brightness

        # 辅星通常不标注亮度，但可以预留接口
        # for star in palace['minor_stars']:
        #     brightness = get_star_brightness(star.name, palace_branch)
        #     if brightness:
        #         star.brightness = brightness


def get_brightness_score(brightness: Optional[Brightness]) -> int:
    """
    获取亮度的数值评分（用于比较）

    Args:
        brightness: 亮度等级

    Returns:
        数值评分 (0-6)，庙最高，陷最低
    """
    scores = {"庙": 6, "旺": 5, "得": 4, "利": 3, "平": 2, "不": 1, "陷": 0, None: 0}

    return scores.get(brightness, 0)


def is_bright(brightness: Optional[Brightness]) -> bool:
    """
    判断星曜是否处于庙旺状态

    Args:
        brightness: 亮度等级

    Returns:
        是否为庙或旺
    """
    return brightness in ["庙", "旺"]


def is_weak(brightness: Optional[Brightness]) -> bool:
    """
    判断星曜是否处于陷弱状态

    Args:
        brightness: 亮度等级

    Returns:
        是否为陷
    """
    return brightness == "陷"



"""
国际化 (i18n) 模块

提供多语言支持，默认语言为简体中文 (zh-CN)
支持的语言：
- zh-CN: 简体中文
- zh-TW: 繁體中文
- en-US: English
- ja-JP: 日本語
- ko-KR: 한국어
- vi-VN: Tiếng Việt
"""

from typing import Dict, Any, Optional

# 当前语言设置
_current_language = "zh-CN"

# 语言资源缓存
_locales: Dict[str, Dict[str, Any]] = {}


def set_language(lang: str) -> None:
    """
    设置当前语言

    Args:
        lang: 语言代码，支持 'zh-CN', 'zh-TW', 'en-US', 'ja-JP', 'ko-KR', 'vi-VN'
              不支持的语言将降级为 'zh-CN'
    """
    global _current_language
    supported = ["zh-CN", "zh-TW", "en-US", "ja-JP", "ko-KR", "vi-VN"]

    # 如果语言不支持，降级到中文，但不报错
    if lang not in supported:
        import warnings

        warnings.warn(
            f"Language '{lang}' is not fully supported yet. Falling back to 'zh-CN'. Supported: {supported}",
            UserWarning,
        )
        lang = "zh-CN"

    _current_language = lang
    _load_locale(lang)


def get_language() -> str:
    """
    获取当前语言

    Returns:
        当前语言代码
    """
    return _current_language


def _load_locale(lang: str) -> None:
    """
    加载语言资源文件

    Args:
        lang: 语言代码
    """
    if lang in _locales:
        return

    try:
        if lang == "zh-CN":
            from .locales import zh_CN

            _locales[lang] = zh_CN.translations
        elif lang == "zh-TW":
            from .locales import zh_TW

            _locales[lang] = zh_TW.translations
        elif lang == "en-US":
            from .locales import en_US

            _locales[lang] = en_US.translations
        elif lang == "ja-JP":
            from .locales import ja_JP

            _locales[lang] = ja_JP.translations
        elif lang == "ko-KR":
            from .locales import ko_KR

            _locales[lang] = ko_KR.translations
        elif lang == "vi-VN":
            from .locales import vi_VN

            _locales[lang] = vi_VN.translations
    except ImportError:
        raise ValueError(f"Language resource not found: {lang}")


def t(key: str, lang: Optional[str] = None) -> str:
    """
    翻译函数

    Args:
        key: 翻译键名（英文键）
        lang: 可选，指定语言。如不指定则使用当前语言

    Returns:
        翻译后的文本
    """
    target_lang = lang or _current_language

    # 确保语言资源已加载
    if target_lang not in _locales:
        _load_locale(target_lang)

    locale = _locales.get(target_lang, {})

    # 支持嵌套键，如 'palaces.soulPalace'
    keys = key.split(".")
    value = locale
    for k in keys:
        if isinstance(value, dict):
            value = value.get(k, key)
        else:
            return key

    return value if isinstance(value, str) else key


def translate_dict(data: Dict[str, Any], lang: Optional[str] = None) -> Dict[str, Any]:
    """
    翻译字典中的值

    Args:
        data: 要翻译的字典
        lang: 可选，指定语言

    Returns:
        翻译后的字典
    """
    result: Dict[str, Any] = {}
    for key, value in data.items():
        if isinstance(value, str):
            result[key] = t(value, lang)
        elif isinstance(value, dict):
            result[key] = translate_dict(value, lang)
        elif isinstance(value, list):
            result[key] = [t(v, lang) if isinstance(v, str) else v for v in value]
        else:
            result[key] = value
    return result


# 默认加载中文
_load_locale("zh-CN")


__all__ = ["set_language", "get_language", "t", "translate_dict"]



"""
iztro-py 基本使用示例

演示如何使用 iztro-py 库创建和查询紫微斗数星盘。
"""

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../src')))

from iztro_py import astro

# ============================================================
# 示例 1：创建星盘（使用阳历日期）
# ============================================================
print("=" * 60)
print("示例 1：创建星盘（使用阳历日期）")
print("=" * 60)

# 创建星盘：2000年8月16日午时出生的男命
chart = astro.by_solar('2000-8-16', 6, '男')

# 查看基本信息
print(f"性别: {chart.gender}")
print(f"阳历: {chart.solar_date}")
print(f"农历: {chart.lunar_date}")
print(f"四柱: {chart.chinese_date}")
print(f"生肖: {chart.zodiac}")
print(f"星座: {chart.sign}")
print(f"五行局: {chart.five_elements_class}")
print()


# ============================================================
# 示例 2：查询命宫和身宫
# ============================================================
print("=" * 60)
print("示例 2：查询命宫和身宫")
print("=" * 60)

# 获取命宫
soul_palace = chart.get_soul_palace()
print(f"命宫: {soul_palace.name}")
print(f"  天干地支: {soul_palace.heavenly_stem} {soul_palace.earthly_branch}")
print(f"  主星: {', '.join(s.name for s in soul_palace.major_stars) or '空宫'}")

# 获取身宫
body_palace = chart.get_body_palace()
print(f"\n身宫: {body_palace.name}")
print()


# ============================================================
# 示例 3：查询指定宫位
# ============================================================
print("=" * 60)
print("示例 3：查询指定宫位")
print("=" * 60)

# 通过索引查询
palace_0 = chart.palace(0)
print(f"索引 0 的宫位: {palace_0.name}")

# 通过中文名称查询
wealth_palace = chart.palace('财帛')
print(f"财帛宫: {wealth_palace.name}")
print(f"  主星: {', '.join(s.name for s in wealth_palace.major_stars) or '空宫'}")

# 通过英文名称查询
career_palace = chart.palace('career')
print(f"事业宫: {career_palace.name}")
print(f"  主星: {', '.join(s.name for s in career_palace.major_stars) or '空宫'}")
print()


# ============================================================
# 示例 4：查询星曜
# ============================================================
print("=" * 60)
print("示例 4：查询星曜")
print("=" * 60)

# 查找紫微星（使用星曜的英文名称）
ziwei = chart.star('ziweiMaj')
if ziwei:
    print(f"紫微星:")
    print(f"  亮度: {ziwei.brightness}")
    print(f"  四化: {ziwei.mutagen or '无'}")

    # 查看所在宫位
    palace = ziwei.palace()
    if palace:
        print(f"  所在宫位: {palace.name}")

    # 查看是否庙旺
    print(f"  是否庙旺: {ziwei.is_bright()}")
else:
    print("未找到紫微星")

# 查找天机星
tianji = chart.star('tianjiMaj')
if tianji:
    print(f"\n天机星:")
    print(f"  亮度: {tianji.brightness}")
    print(f"  四化: {tianji.mutagen or '无'}")
    palace = tianji.palace()
    if palace:
        print(f"  所在宫位: {palace.name}")
print()


# ============================================================
# 示例 5：宫位星曜查询
# ============================================================
print("=" * 60)
print("示例 5：宫位星曜查询")
print("=" * 60)

# 检查命宫是否包含特定星曜
soul = chart.get_soul_palace()
print(f"命宫包含天机星: {soul.has(['tianjiMaj'])}")
print(f"命宫包含紫微星: {soul.has(['ziweiMaj'])}")

# 检查是否有化禄星
print(f"命宫有化禄: {soul.has_mutagen('禄')}")

# 检查是否为空宫
print(f"命宫是否空宫: {soul.is_empty()}")
print()


# ============================================================
# 示例 6：三方四正查询
# ============================================================
print("=" * 60)
print("示例 6：三方四正查询")
print("=" * 60)

# 获取命宫的三方四正
soul = chart.get_soul_palace()
surpalaces = chart.surrounded_palaces(soul.index)

if surpalaces:
    print(f"命宫三方四正:")
    for p in surpalaces.all_palaces():
        major = ', '.join(s.name for s in p.major_stars) or '空'
        print(f"  {p.name}: {major}")

    # 检查三方四正是否有特定星曜
    print(f"\n三方四正是否有紫微: {surpalaces.have(['ziweiMaj'])}")
    print(f"三方四正是否有化禄: {surpalaces.have_mutagen('禄')}")
print()


# ============================================================
# 示例 7：遍历所有宫位
# ============================================================
print("=" * 60)
print("示例 7：遍历所有宫位")
print("=" * 60)

for i in range(12):
    palace = chart.palace(i)
    if palace:
        # 标记命宫和身宫
        markers = []
        if palace.is_original_palace:
            markers.append("命")
        if palace.is_body_palace:
            markers.append("身")
        marker = f"[{'/'.join(markers)}]" if markers else ""

        # 获取主星
        major = ', '.join(s.name for s in palace.major_stars) or '空'

        # 获取四化星
        mutagen_stars = [
            f"{s.name}化{s.mutagen}"
            for s in palace.major_stars + palace.minor_stars
            if s.mutagen
        ]
        mutagen_str = ', '.join(mutagen_stars) if mutagen_stars else ''

        print(f"{i:2d}. {palace.name:15s}{marker:8s} {major:25s} {mutagen_str}")
print()


# ============================================================
# 示例 8：使用农历日期
# ============================================================
print("=" * 60)
print("示例 8：使用农历日期")
print("=" * 60)

# 使用农历日期创建星盘
chart2 = astro.by_lunar('2000-7-17', 6, '男')
print(f"阳历: {chart2.solar_date}")
print(f"农历: {chart2.lunar_date}")
print(f"生肖: {chart2.zodiac}")
print()


# ============================================================
# 示例 9：便捷函数
# ============================================================
print("=" * 60)
print("示例 9：便捷函数")
print("=" * 60)

# 直接获取生肖
zodiac = astro.get_zodiac_by_solar_date('2000-8-16')
print(f"2000-8-16 的生肖: {zodiac}")

# 直接获取星座
sign = astro.get_sign_by_solar_date('2000-8-16')
print(f"2000-8-16 的星座: {sign}")
print()


print("=" * 60)
print("示例完成！")
print("=" * 60)



# -*- coding: utf-8 -*-
"""English language resources"""

translations = {
    "palaces": {
        "soulPalace": "Soul",
        "parentsPalace": "Parents",
        "spiritPalace": "Spirit",
        "propertyPalace": "Property",
        "careerPalace": "Career",
        "friendsPalace": "Friends",
        "surfacePalace": "Surface",
        "healthPalace": "Health",
        "wealthPalace": "Wealth",
        "childrenPalace": "Children",
        "spousePalace": "Spouse",
        "siblingsPalace": "Siblings",
    },
    "stars": {
        "major": {
            "ziweiMaj": "Ziwei",
            "tianjiMaj": "Tianji",
            "taiyangMaj": "Taiyang",
            "wuquMaj": "Wuqu",
            "tiantongMaj": "Tiantong",
            "lianzhenMaj": "Lianzhen",
            "tianfuMaj": "Tianfu",
            "taiyinMaj": "Taiyin",
            "tanlangMaj": "Tanlang",
            "jumenMaj": "Jumen",
            "tianxiangMaj": "Tianxiang",
            "tianliangMaj": "Tianliang",
            "qishaMaj": "Qisha",
            "pojunMaj": "Pojun",
        },
        "minor": {
            "zuofuMin": "Zuofu",
            "youbiMin": "Youbi",
            "wenchangMin": "Wenchang",
            "wenquMin": "Wenqu",
            "tiankuiMin": "Tiankui",
            "tianyueMin": "Tianyue",
            "huoxingMin": "Huoxing",
            "lingxingMin": "Lingxing",
            "dikongMin": "Dikong",
            "dijieMin": "Dijie",
            "lucunMin": "Lucun",
            "qingyangMin": "Qingyang",
            "tuoluoMin": "Tuoluo",
            "tianmaMin": "Tianma",
        },
    },
    "heavenlyStem": {
        "jiaHeavenly": "Jia",
        "yiHeavenly": "Yi",
        "bingHeavenly": "Bing",
        "dingHeavenly": "Ding",
        "wuHeavenly": "Wu",
        "jiHeavenly": "Ji",
        "gengHeavenly": "Geng",
        "xinHeavenly": "Xin",
        "renHeavenly": "Ren",
        "guiHeavenly": "Gui",
    },
    "earthlyBranch": {
        "ziEarthly": "Zi",
        "chouEarthly": "Chou",
        "yinEarthly": "Yin",
        "maoEarthly": "Mao",
        "chenEarthly": "Chen",
        "siEarthly": "Si",
        "wuEarthly": "Wu",
        "weiEarthly": "Wei",
        "shenEarthly": "Shen",
        "youEarthly": "You",
        "xuEarthly": "Xu",
        "haiEarthly": "Hai",
    },
    "brightness": {
        "miao": "Temple",
        "wang": "Prosperous",
        "de": "Favorable",
        "li": "Beneficial",
        "ping": "Neutral",
        "bu": "Unfavorable",
        "xian": "Trapped",
    },
}



"""
FunctionalPalace class - Palace with functional methods

Provides a rich API for querying palace properties and stars.
"""

from typing import Optional, List, TYPE_CHECKING
from iztro_py.data.types import Palace, StarName, Mutagen
from iztro_py.astro.functional_star import FunctionalStar

if TYPE_CHECKING:
    from iztro_py.astro.functional_astrolabe import FunctionalAstrolabe
    from iztro_py.astro.functional_surpalaces import FunctionalSurpalaces


class FunctionalPalace(Palace):
    """
    功能增强的宫位类

    继承自Palace，添加了星曜查询方法和关联星盘的能力
    """

    def __init__(self, palace: Palace):
        """
        初始化FunctionalPalace

        Args:
            palace: 基础Palace对象
        """
        # 转换星曜为FunctionalStar
        major_stars = [FunctionalStar(s) for s in palace.major_stars]
        minor_stars = [FunctionalStar(s) for s in palace.minor_stars]
        adjective_stars = [FunctionalStar(s) for s in palace.adjective_stars]

        super().__init__(
            index=palace.index,
            name=palace.name,
            is_body_palace=palace.is_body_palace,
            is_original_palace=palace.is_original_palace,
            heavenly_stem=palace.heavenly_stem,
            earthly_branch=palace.earthly_branch,
            major_stars=major_stars,
            minor_stars=minor_stars,
            adjective_stars=adjective_stars,
            changsheng12=palace.changsheng12,
            boshi12=palace.boshi12,
            jiangqian12=palace.jiangqian12,
            suiqian12=palace.suiqian12,
            decadal=palace.decadal,
            ages=palace.ages,
        )

        self._astrolabe: Optional["FunctionalAstrolabe"] = None

        # 设置星曜的宫位引用
        for star in self.major_stars + self.minor_stars + self.adjective_stars:
            star.set_palace(self)

    def set_astrolabe(self, astrolabe: "FunctionalAstrolabe") -> None:
        """
        设置宫位所属的星盘

        Args:
            astrolabe: 星盘对象
        """
        self._astrolabe = astrolabe

    def astrolabe(self) -> Optional["FunctionalAstrolabe"]:
        """
        获取宫位所属的星盘

        Returns:
            星盘对象，如果未设置则返回None
        """
        return self._astrolabe

    def has(self, stars: List[StarName]) -> bool:
        """
        判断宫位是否包含所有指定的星曜

        Args:
            stars: 星曜名称列表

        Returns:
            是否包含所有星曜

        Example:
            >>> palace.has(['紫微', '天府'])
        """
        all_stars = self.major_stars + self.minor_stars + self.adjective_stars
        star_names = [s.name for s in all_stars]

        return all(star in star_names for star in stars)

    def has_one_of(self, stars: List[StarName]) -> bool:
        """
        判断宫位是否包含任一指定的星曜

        Args:
            stars: 星曜名称列表

        Returns:
            是否包含任一星曜

        Example:
            >>> palace.has_one_of(['紫微', '天府'])
        """
        all_stars = self.major_stars + self.minor_stars + self.adjective_stars
        star_names = [s.name for s in all_stars]

        return any(star in star_names for star in stars)

    def not_have(self, stars: List[StarName]) -> bool:
        """
        判断宫位是否不包含任何指定的星曜

        Args:
            stars: 星曜名称列表

        Returns:
            是否都不包含

        Example:
            >>> palace.not_have(['火星', '铃星'])
        """
        all_stars = self.major_stars + self.minor_stars + self.adjective_stars
        star_names = [s.name for s in all_stars]

        return all(star not in star_names for star in stars)

    def has_mutagen(self, mutagen: Mutagen) -> bool:
        """
        判断宫位是否包含指定四化的星曜

        Args:
            mutagen: 四化类型 ('禄'/'权'/'科'/'忌')

        Returns:
            是否包含

        Example:
            >>> palace.has_mutagen('禄')
        """
        all_stars = self.major_stars + self.minor_stars + self.adjective_stars
        return any(s.mutagen == mutagen for s in all_stars)

    def not_have_mutagen(self, mutagen: Mutagen) -> bool:
        """
        判断宫位是否不包含指定四化的星曜

        Args:
            mutagen: 四化类型

        Returns:
            是否不包含

        Example:
            >>> palace.not_have_mutagen('忌')
        """
        return not self.has_mutagen(mutagen)

    def is_empty(self, exclude_stars: Optional[List[StarName]] = None) -> bool:
        """
        判断宫位是否为空宫（无主星）

        Args:
            exclude_stars: 要排除的星曜列表（可选）

        Returns:
            是否为空宫

        Example:
            >>> palace.is_empty()
            >>> palace.is_empty(exclude_stars=['禄存', '天马'])
        """
        major_stars = self.major_stars

        if exclude_stars:
            major_stars = [s for s in major_stars if s.name not in exclude_stars]

        return len(major_stars) == 0

    def get_star(self, star_name: StarName) -> Optional[FunctionalStar]:
        """
        获取指定名称的星曜对象

        Args:
            star_name: 星曜名称

        Returns:
            星曜对象，如果不存在则返回None
        """
        all_stars = self.major_stars + self.minor_stars + self.adjective_stars

        for star in all_stars:
            if star.name == star_name:
                return star

        return None

    def __str__(self) -> str:
        """字符串表示"""
        markers = []
        if self.is_original_palace:
            markers.append("命")
        if self.is_body_palace:
            markers.append("身")

        marker_str = f"[{'/'.join(markers)}]" if markers else ""

        major_str = ", ".join(str(s) for s in self.major_stars) if self.major_stars else "空宫"

        return f"{self.name}{marker_str}: {major_str}"

    def __repr__(self) -> str:
        return f"FunctionalPalace(index={self.index}, name={self.name})"



# -*- coding: utf-8 -*-
"""日本語言語リソース (Japanese language resources)"""

translations = {
    "palaces": {
        "soulPalace": "命宮",
        "parentsPalace": "父母宮",
        "spiritPalace": "福徳宮",
        "propertyPalace": "田宅宮",
        "careerPalace": "官禄宮",
        "friendsPalace": "交友宮",
        "surfacePalace": "遷移宮",
        "healthPalace": "疾厄宮",
        "wealthPalace": "財帛宮",
        "childrenPalace": "子女宮",
        "spousePalace": "夫妻宮",
        "siblingsPalace": "兄弟宮",
    },
    "stars": {
        "major": {
            "ziweiMaj": "紫微",
            "tianjiMaj": "天機",
            "taiyangMaj": "太陽",
            "wuquMaj": "武曲",
            "tiantongMaj": "天同",
            "lianzhenMaj": "廉貞",
            "tianfuMaj": "天府",
            "taiyinMaj": "太陰",
            "tanlangMaj": "貪狼",
            "jumenMaj": "巨門",
            "tianxiangMaj": "天相",
            "tianliangMaj": "天梁",
            "qishaMaj": "七殺",
            "pojunMaj": "破軍",
        },
        "minor": {
            "zuofuMin": "左輔",
            "youbiMin": "右弼",
            "wenchangMin": "文昌",
            "wenquMin": "文曲",
            "tiankuiMin": "天魁",
            "tianyueMin": "天鉞",
            "huoxingMin": "火星",
            "lingxingMin": "鈴星",
            "dikongMin": "地空",
            "dijieMin": "地劫",
            "lucunMin": "禄存",
            "qingyangMin": "擎羊",
            "tuoluoMin": "陀羅",
            "tianmaMin": "天馬",
        },
    },
    "heavenlyStem": {
        "jiaHeavenly": "甲",
        "yiHeavenly": "乙",
        "bingHeavenly": "丙",
        "dingHeavenly": "丁",
        "wuHeavenly": "戊",
        "jiHeavenly": "己",
        "gengHeavenly": "庚",
        "xinHeavenly": "辛",
        "renHeavenly": "壬",
        "guiHeavenly": "癸",
    },
    "earthlyBranch": {
        "ziEarthly": "子",
        "chouEarthly": "丑",
        "yinEarthly": "寅",
        "maoEarthly": "卯",
        "chenEarthly": "辰",
        "siEarthly": "巳",
        "wuEarthly": "午",
        "weiEarthly": "未",
        "shenEarthly": "申",
        "youEarthly": "酉",
        "xuEarthly": "戌",
        "haiEarthly": "亥",
    },
    "brightness": {
        "miao": "廟",
        "wang": "旺",
        "de": "得",
        "li": "利",
        "ping": "平",
        "bu": "不",
        "xian": "陷",
    },
}



"""
Heavenly Stems (天干) data and properties

Contains detailed information about the ten heavenly stems including
yin-yang, five elements, clashes, and mutagen (四化) configurations.
"""

from typing import Dict, List, Optional
from iztro_py.data.types import HeavenlyStemName, YinYang, FiveElements, StarName, Mutagen


class HeavenlyStem:
    """天干数据类"""

    def __init__(
        self,
        yin_yang: YinYang,
        five_elements: FiveElements,
        crash: HeavenlyStemName,
        mutagen: List[StarName],
    ):
        self.yin_yang = yin_yang
        self.five_elements = five_elements
        self.crash = crash  # 相冲的天干
        self.mutagen = mutagen  # 四化：禄、权、科、忌


# ============================================================================
# Heavenly Stems Configuration
# ============================================================================

HEAVENLY_STEMS_CONFIG: Dict[HeavenlyStemName, HeavenlyStem] = {
    # 甲天干
    "jiaHeavenly": HeavenlyStem(
        yin_yang="阳",
        five_elements="木",
        crash="gengHeavenly",
        mutagen=["lianzhenMaj", "pojunMaj", "wuquMaj", "taiyangMaj"],  # 廉禄、破权、武科、阳忌
    ),
    # 乙天干
    "yiHeavenly": HeavenlyStem(
        yin_yang="阴",
        five_elements="木",
        crash="xinHeavenly",
        mutagen=["tianjiMaj", "tianliangMaj", "ziweiMaj", "taiyinMaj"],  # 机禄、梁权、紫科、月忌
    ),
    # 丙天干
    "bingHeavenly": HeavenlyStem(
        yin_yang="阳",
        five_elements="火",
        crash="renHeavenly",
        mutagen=[
            "tiantongMaj",
            "tianjiMaj",
            "wenchangMin",
            "lianzhenMaj",
        ],  # 同禄、机权、昌科、廉忌
    ),
    # 丁天干
    "dingHeavenly": HeavenlyStem(
        yin_yang="阴",
        five_elements="火",
        crash="guiHeavenly",
        mutagen=["taiyinMaj", "tiantongMaj", "tianjiMaj", "jumenMaj"],  # 阴禄、同权、机科、巨忌
    ),
    # 戊天干
    "wuHeavenly": HeavenlyStem(
        yin_yang="阳",
        five_elements="土",
        crash="jiaHeavenly",
        mutagen=["tanlangMaj", "taiyinMaj", "youbiMin", "tianjiMaj"],  # 贪禄、月权、弼科、机忌
    ),
    # 己天干
    "jiHeavenly": HeavenlyStem(
        yin_yang="阴",
        five_elements="土",
        crash="yiHeavenly",
        mutagen=["wuquMaj", "tanlangMaj", "tianliangMaj", "wenquMin"],  # 武禄、贪权、梁科、曲忌
    ),
    # 庚天干
    "gengHeavenly": HeavenlyStem(
        yin_yang="阳",
        five_elements="金",
        crash="jiaHeavenly",
        mutagen=["taiyangMaj", "wuquMaj", "taiyinMaj", "tiantongMaj"],  # 阳禄、武权、阴科、同忌
    ),
    # 辛天干
    "xinHeavenly": HeavenlyStem(
        yin_yang="阴",
        five_elements="金",
        crash="yiHeavenly",
        mutagen=["jumenMaj", "taiyangMaj", "wenquMin", "wenchangMin"],  # 巨禄、阳权、曲科、昌忌
    ),
    # 壬天干
    "renHeavenly": HeavenlyStem(
        yin_yang="阳",
        five_elements="水",
        crash="bingHeavenly",
        mutagen=["tianliangMaj", "ziweiMaj", "zuofuMin", "wuquMaj"],  # 梁禄、紫权、左科、武忌
    ),
    # 癸天干
    "guiHeavenly": HeavenlyStem(
        yin_yang="阴",
        five_elements="水",
        crash="dingHeavenly",
        mutagen=["pojunMaj", "jumenMaj", "taiyinMaj", "tanlangMaj"],  # 破禄、巨权、阴科、贪忌
    ),
}


# ============================================================================
# Helper Functions
# ============================================================================


def get_mutagen(heavenly_stem: HeavenlyStemName) -> List[StarName]:
    """
    获取指定天干的四化星

    Args:
        heavenly_stem: 天干名称

    Returns:
        四化星列表 [禄, 权, 科, 忌]
    """
    return HEAVENLY_STEMS_CONFIG[heavenly_stem].mutagen


def get_mutagen_type(heavenly_stem: HeavenlyStemName, star_name: StarName) -> Optional[Mutagen]:
    """
    获取指定星耀在指定天干下的四化类型

    Args:
        heavenly_stem: 天干名称
        star_name: 星耀名称

    Returns:
        四化类型：'禄'、'权'、'科'、'忌'，如果不是四化星则返回 None
    """
    mutagen_stars = HEAVENLY_STEMS_CONFIG[heavenly_stem].mutagen
    mutagen_types: List[Mutagen] = ["禄", "权", "科", "忌"]

    if star_name in mutagen_stars:
        index = mutagen_stars.index(star_name)
        return mutagen_types[index]

    return None


def get_yin_yang(heavenly_stem: HeavenlyStemName) -> YinYang:
    """
    获取天干的阴阳属性

    Args:
        heavenly_stem: 天干名称

    Returns:
        阴阳属性
    """
    return HEAVENLY_STEMS_CONFIG[heavenly_stem].yin_yang


def get_five_elements(heavenly_stem: HeavenlyStemName) -> FiveElements:
    """
    获取天干的五行属性

    Args:
        heavenly_stem: 天干名称

    Returns:
        五行属性
    """
    return HEAVENLY_STEMS_CONFIG[heavenly_stem].five_elements


def get_crash(heavenly_stem: HeavenlyStemName) -> HeavenlyStemName:
    """
    获取相冲的天干

    Args:
        heavenly_stem: 天干名称

    Returns:
        相冲的天干名称
    """
    return HEAVENLY_STEMS_CONFIG[heavenly_stem].crash



"""
Data module: Core data definitions including types, constants, and reference data.
"""

from iztro_py.data import types, constants, heavenly_stems, earthly_branches

from iztro_py.data.types import (
    Language,
    YinYang,
    FiveElements,
    GenderName,
    Mutagen,
    Brightness,
    Scope,
    StarType,
    ChineseTime,
    HeavenlyStemName,
    EarthlyBranchName,
    PalaceName,
    StarName,
    MajorStarName,
    MinorStarName,
    AdjectiveStarName,
    FiveElementsClass,
    Star,
    Decadal,
    Palace,
    SoulAndBody,
    LunarDate,
    HeavenlyStemAndEarthlyBranchDate,
    Astrolabe,
    SurroundedPalaces,
    HoroscopeItem,
    Horoscope,
    Config,
    AstrolabeOptions,
)

from iztro_py.data.constants import (
    CHINESE_TIME,
    TIME_RANGE,
    HEAVENLY_STEMS,
    EARTHLY_BRANCHES,
    PALACES,
    MAJOR_STARS,
    MINOR_STARS,
    MUTAGEN,
    ZIWEI_GROUP,
    TIANFU_GROUP,
    TIGER_RULE,
    RAT_RULE,
    FIVE_ELEMENTS_CLASS_LOOKUP,
    ZIWEI_START_POSITIONS,
    fix_index,
    get_opposite_index,
    get_surrounded_indices,
)

from iztro_py.data.heavenly_stems import (
    HEAVENLY_STEMS_CONFIG,
    get_mutagen,
    get_mutagen_type,
)

from iztro_py.data.earthly_branches import (
    EARTHLY_BRANCHES_CONFIG,
    get_soul_star,
    get_body_star,
)

__all__ = [
    # Submodules
    "types",
    "constants",
    "heavenly_stems",
    "earthly_branches",
    # Type exports
    "Language",
    "YinYang",
    "FiveElements",
    "GenderName",
    "Mutagen",
    "Brightness",
    "Scope",
    "StarType",
    "ChineseTime",
    "HeavenlyStemName",
    "EarthlyBranchName",
    "PalaceName",
    "StarName",
    "MajorStarName",
    "MinorStarName",
    "AdjectiveStarName",
    "FiveElementsClass",
    "Star",
    "Decadal",
    "Palace",
    "SoulAndBody",
    "LunarDate",
    "HeavenlyStemAndEarthlyBranchDate",
    "Astrolabe",
    "SurroundedPalaces",
    "HoroscopeItem",
    "Horoscope",
    "Config",
    "AstrolabeOptions",
    # Constants
    "CHINESE_TIME",
    "TIME_RANGE",
    "HEAVENLY_STEMS",
    "EARTHLY_BRANCHES",
    "PALACES",
    "MAJOR_STARS",
    "MINOR_STARS",
    "MUTAGEN",
    "ZIWEI_GROUP",
    "TIANFU_GROUP",
    "TIGER_RULE",
    "RAT_RULE",
    "FIVE_ELEMENTS_CLASS_LOOKUP",
    "ZIWEI_START_POSITIONS",
    "fix_index",
    "get_opposite_index",
    "get_surrounded_indices",
    # Heavenly Stems
    "HEAVENLY_STEMS_CONFIG",
    "get_mutagen",
    "get_mutagen_type",
    # Earthly Branches
    "EARTHLY_BRANCHES_CONFIG",
    "get_soul_star",
    "get_body_star",
]



"""
Major stars placement for iztro-py

Functions for placing the 14 major stars (主星) into palaces.
"""

from typing import Any, Dict, List
from iztro_py.data.types import Star, FiveElementsClass
from iztro_py.star.location import get_major_star_positions


def place_major_stars(
    palaces: List[Dict[str, Any]],
    arg1,
    arg2,
) -> None:
    """
    将14颗主星安置到宫位中

    Args:
        palaces: 宫位列表
        arg1: 兼容两种调用方式：
              - (FiveElementsClass, lunar_day)
              - (ziwei_index: int, tianfu_index: int)
        arg2: 同上

    Note:
        直接修改palaces列表，不返回值
    """
    from iztro_py.data.constants import EARTHLY_BRANCHES

    # 兼容旧API：如果第一个参数是 FiveElementsClass，则按旧算法计算索引
    if isinstance(arg1, FiveElementsClass):
        from iztro_py.star.location import get_ziwei_index, get_tianfu_index

        five_class: FiveElementsClass = arg1
        lunar_day: int = arg2
        ziwei_index = get_ziwei_index(five_class, lunar_day)
        tianfu_index = get_tianfu_index(ziwei_index)
    else:
        ziwei_index = int(arg1)
        tianfu_index = int(arg2)

    # 获取所有主星位置（地支索引）
    star_positions = get_major_star_positions(ziwei_index, tianfu_index)

    # 将星曜放置到对应宫位
    # 注意：star_positions 中的索引是地支索引，需要转换为宫位索引
    for star_name, earthly_branch_index in star_positions.items():
        star = Star(
            name=star_name,
            type="major",
            scope="origin",
            brightness=None,  # 后续计算
            mutagen=None,  # 后续计算
        )

        # 查找具有该地支的宫位
        target_branch = EARTHLY_BRANCHES[earthly_branch_index]
        for palace in palaces:
            if palace["earthly_branch"] == target_branch:
                palace["major_stars"].append(star)
                break


def get_major_stars_in_palace(palace: dict) -> List[Star]:
    """
    获取指定宫位中的主星列表

    Args:
        palace: 宫位字典

    Returns:
        主星列表
    """
    return palace.get("major_stars", [])


def has_major_star(palace: dict, star_name: str) -> bool:
    """
    判断宫位是否包含指定主星

    Args:
        palace: 宫位字典
        star_name: 星曜名称

    Returns:
        是否包含
    """
    major_stars = get_major_stars_in_palace(palace)
    return any(star.name == star_name for star in major_stars)



"""
Palace positioning algorithms for iztro-py

Contains functions for calculating palace positions, especially
the soul palace (命宫) and body palace (身宫).
"""

from typing import Any, Dict, List
from iztro_py.data.types import SoulAndBody, HeavenlyStemName, EarthlyBranchName
from iztro_py.data.constants import HEAVENLY_STEMS, EARTHLY_BRANCHES, TIGER_RULE, fix_index


def get_soul_and_body(
    lunar_month: int, time_index: int, year_stem: HeavenlyStemName
) -> SoulAndBody:
    """
    计算命宫和身宫的位置

    算法：
    1. 命宫：寅起正月，顺数至生月，再逆数生时
    2. 身宫：从生月宫位顺数生时

    Args:
        lunar_month: 农历月份 (1-12)
        time_index: 时辰索引 (0-12)
        year_stem: 年干（用于计算命宫天干）

    Returns:
        SoulAndBody对象，包含命宫和身宫的索引及天干地支
    """
    # 1. 寅起正月，顺数至生月
    # 寅 = 索引2，正月 = lunar_month 1
    # 生月宫位索引 = (2 + lunar_month - 1) % 12
    birth_month_palace_index = fix_index(2 + lunar_month - 1)

    # 2. 从生月宫位逆数生时，得到命宫
    # 时辰索引转换：0(早子时)和12(晚子时)都对应子时，其他时辰直接对应
    actual_time_index = 0 if time_index == 12 else time_index

    # 逆数：减去时辰数
    soul_index = fix_index(birth_month_palace_index - actual_time_index)

    # 3. 从生月宫位顺数生时，得到身宫
    body_index = fix_index(birth_month_palace_index + actual_time_index)

    # 4. 计算命宫地支
    earthly_branch_of_soul = EARTHLY_BRANCHES[soul_index]

    # 5. 计算命宫天干（使用五虎遁）
    # 先找到寅宫的天干
    # 五虎遁：根据年干确定寅月（正月）的天干
    yin_month_stem = TIGER_RULE[year_stem]
    yin_month_stem_index = HEAVENLY_STEMS.index(yin_month_stem)

    # 寅宫索引是2，命宫索引是soul_index
    # 计算从寅宫到命宫的天干偏移
    # 地支顺序：子丑寅卯辰巳午未申酉戌亥
    # 从寅(2)到命宫的距离
    if soul_index >= 2:
        offset = soul_index - 2
    else:
        offset = soul_index + 12 - 2

    # 命宫天干
    heavenly_stem_of_soul_index = fix_index(yin_month_stem_index + offset, 10)
    heavenly_stem_of_soul = HEAVENLY_STEMS[heavenly_stem_of_soul_index]

    return SoulAndBody(
        soul_index=soul_index,
        body_index=body_index,
        heavenly_stem_of_soul=heavenly_stem_of_soul,
        earthly_branch_of_soul=earthly_branch_of_soul,
    )


def get_palace_heavenly_stem(
    palace_index: int, soul_palace_index: int, soul_palace_stem: HeavenlyStemName
) -> HeavenlyStemName:
    """
    根据命宫天干推算其他宫位的天干

    Args:
        palace_index: 目标宫位索引
        soul_palace_index: 命宫索引
        soul_palace_stem: 命宫天干

    Returns:
        目标宫位的天干
    """
    soul_stem_index = HEAVENLY_STEMS.index(soul_palace_stem)

    # 计算从命宫到目标宫位的距离
    if palace_index >= soul_palace_index:
        offset = palace_index - soul_palace_index
    else:
        offset = palace_index + 12 - soul_palace_index

    # 目标宫位天干索引
    target_stem_index = fix_index(soul_stem_index + offset, 10)

    return HEAVENLY_STEMS[target_stem_index]


def get_palace_earthly_branch(palace_index: int) -> EarthlyBranchName:
    """
    根据宫位索引获取地支

    宫位地支固定：
    0(命宫) - 根据命宫定位算法确定
    其他宫位按地支顺序排列

    Args:
        palace_index: 宫位索引 (0-11)

    Returns:
        地支名称
    """
    return EARTHLY_BRANCHES[palace_index]


def get_body_palace_index(soul_index: int, body_index: int) -> int:
    """
    确定身宫所在的宫位

    身宫会落在某个宫位上，该宫位标记为身宫

    Args:
        soul_index: 命宫索引
        body_index: 身宫索引（通过算法计算出的地支位置）

    Returns:
        身宫所在的宫位索引
    """
    # 身宫的索引就是其地支位置相对于命宫的偏移
    # 因为十二宫固定从命宫开始：命宫(0)、父母(1)、福德(2)...
    # 而身宫的地支位置是通过"从生月顺数生时"得到的

    # 直接返回body_index，因为宫位是从命宫开始按地支顺序排列的
    return body_index


def calculate_palace_ages(
    palace_index: int, soul_palace_index: int, five_elements_class_value: int, is_forward: bool
) -> List[int]:
    """
    计算宫位的小限年龄数组

    小限从命宫开始，每年走一宫
    顺逆根据性别和年支阴阳决定

    Args:
        palace_index: 宫位索引
        soul_palace_index: 命宫索引
        five_elements_class_value: 五行局数值 (2-6)
        is_forward: 是否顺行

    Returns:
        该宫位对应的年龄列表
    """
    ages = []

    # 小限从命宫开始，起始年龄 = 五行局数值
    start_age = five_elements_class_value

    # 计算当前宫位是从命宫数起的第几个宫位
    if is_forward:
        # 顺行
        offset = (palace_index - soul_palace_index) % 12
    else:
        # 逆行
        offset = (soul_palace_index - palace_index) % 12

    # 该宫位对应的年龄：起始年龄 + 偏移，然后每隔12年一次
    first_age = start_age + offset

    # 生成年龄列表（通常到120岁）
    for age in range(first_age, 121, 12):
        ages.append(age)

    return ages


def initialize_palaces(soul_and_body: SoulAndBody) -> List[Dict[str, Any]]:
    """
    初始化十二宫位的基础信息

    Args:
        soul_and_body: 命身宫位置信息

    Returns:
        包含12个宫位基础信息的列表
    """
    from iztro_py.data.constants import PALACES

    palaces = []

    # 重要：宫位地支是从命宫的地支开始，按地支顺序排列
    # 例如：命宫在亥 -> 父母宫在子 -> 福德宫在丑 -> ...
    soul_index = soul_and_body.soul_index
    body_index = soul_and_body.body_index

    # 计算身宫在宫位序列中的相对索引（以命宫所在宫位为0）
    body_palace_rel_index = fix_index(body_index - soul_index)

    for i in range(12):
        # 计算这个宫位对应的地支索引
        # 从命宫开始，i=0是命宫，i=1是父母宫等
        earthly_branch_index = fix_index(soul_index + i)

        # 判断该宫位是否为身宫（相对命宫的序列位置等于身宫相对索引）
        is_body = i == body_palace_rel_index

        palace: Dict[str, Any] = {
            "index": i,
            "name": PALACES[i],
            "is_body_palace": is_body,
            "is_original_palace": (i == 0),  # 第0个宫位总是命宫
            "earthly_branch": EARTHLY_BRANCHES[earthly_branch_index],
            "heavenly_stem": get_palace_heavenly_stem(
                earthly_branch_index, soul_and_body.soul_index, soul_and_body.heavenly_stem_of_soul
            ),
            "major_stars": [],
            "minor_stars": [],
            "adjective_stars": [],
            "changsheng12": None,
            "boshi12": None,
            "jiangqian12": None,
            "suiqian12": None,
            "decadal": None,
            "ages": [],
        }

        palaces.append(palace)

    return palaces



"""
Earthly Branches (地支) data and properties

Contains detailed information about the twelve earthly branches including
yin-yang, five elements, clashes, soul/body stars, and health associations.
"""

from typing import Dict
from iztro_py.data.types import EarthlyBranchName, YinYang, FiveElements, StarName


class EarthlyBranch:
    """地支数据类"""

    def __init__(
        self,
        yin_yang: YinYang,
        five_elements: FiveElements,
        crash: EarthlyBranchName,
        soul: StarName,  # 命主
        body: StarName,  # 身主
        inside: str,  # 内脏
        outside: str,  # 外部
        health_tip: str,  # 健康提示
    ):
        self.yin_yang = yin_yang
        self.five_elements = five_elements
        self.crash = crash  # 相冲的地支
        self.soul = soul  # 命主星
        self.body = body  # 身主星
        self.inside = inside
        self.outside = outside
        self.health_tip = health_tip


# ============================================================================
# Earthly Branches Configuration
# ============================================================================

EARTHLY_BRANCHES_CONFIG: Dict[EarthlyBranchName, EarthlyBranch] = {
    # 子地支
    "ziEarthly": EarthlyBranch(
        yin_yang="阳",
        five_elements="水",
        crash="wuEarthly",
        soul="tanlangMaj",  # 贪狼
        body="huoxingMin",  # 火星
        inside="胆",
        outside="下体",
        health_tip="生殖系统、膀胱、尿道之疾病",
    ),
    # 丑地支
    "chouEarthly": EarthlyBranch(
        yin_yang="阴",
        five_elements="土",
        crash="weiEarthly",
        soul="jumenMaj",  # 巨门
        body="tianxiangMaj",  # 天相
        inside="肚",
        outside="手足",
        health_tip="脾胃、肠道之疾病",
    ),
    # 寅地支
    "yinEarthly": EarthlyBranch(
        yin_yang="阳",
        five_elements="木",
        crash="shenEarthly",
        soul="lucunMin",  # 禄存
        body="tianliangMaj",  # 天梁
        inside="肝",
        outside="四肢",
        health_tip="肝胆、四肢筋骨之疾病",
    ),
    # 卯地支
    "maoEarthly": EarthlyBranch(
        yin_yang="阴",
        five_elements="木",
        crash="youEarthly",
        soul="wenquMin",  # 文曲（与 iztro 保持一致）
        body="tiantongMaj",  # 天同
        inside="肝",
        outside="胸背",
        health_tip="肝胆、神经系统之疾病",
    ),
    # 辰地支
    "chenEarthly": EarthlyBranch(
        yin_yang="阳",
        five_elements="土",
        crash="xuEarthly",
        soul="lianzhenMaj",  # 廉贞
        body="wenchangMin",  # 文昌
        inside="胸",
        outside="头面",
        health_tip="脾胃、胸部、皮肤之疾病",
    ),
    # 巳地支
    "siEarthly": EarthlyBranch(
        yin_yang="阴",
        five_elements="火",
        crash="haiEarthly",
        soul="wuquMaj",  # 武曲
        body="tianjiMaj",  # 天机
        inside="心",
        outside="面部",
        health_tip="心脏、血液循环之疾病",
    ),
    # 午地支
    "wuEarthly": EarthlyBranch(
        yin_yang="阳",
        five_elements="火",
        crash="ziEarthly",
        soul="pojunMaj",  # 破军
        body="huoxingMin",  # 火星
        inside="心",
        outside="眼目",
        health_tip="心脏、眼睛、血压之疾病",
    ),
    # 未地支
    "weiEarthly": EarthlyBranch(
        yin_yang="阴",
        five_elements="土",
        crash="chouEarthly",
        soul="wuquMaj",  # 武曲
        body="tianxiangMaj",  # 天相
        inside="脾胃",
        outside="颈项",
        health_tip="脾胃、颈部之疾病",
    ),
    # 申地支
    "shenEarthly": EarthlyBranch(
        yin_yang="阳",
        five_elements="金",
        crash="yinEarthly",
        soul="lianzhenMaj",  # 廉贞
        body="tianliangMaj",  # 天梁
        inside="肺",
        outside="脊柱",
        health_tip="呼吸系统、脊柱之疾病",
    ),
    # 酉地支
    "youEarthly": EarthlyBranch(
        yin_yang="阴",
        five_elements="金",
        crash="maoEarthly",
        soul="wenquMin",  # 文曲
        body="tiantongMaj",  # 天同
        inside="肺",
        outside="咽喉",
        health_tip="呼吸系统、咽喉之疾病",
    ),
    # 戌地支
    "xuEarthly": EarthlyBranch(
        yin_yang="阳",
        five_elements="土",
        crash="chenEarthly",
        soul="lucunMin",  # 禄存
        body="wenchangMin",  # 文昌
        inside="命门",
        outside="腿足",
        health_tip="胃部、腿足、关节之疾病",
    ),
    # 亥地支
    "haiEarthly": EarthlyBranch(
        yin_yang="阴",
        five_elements="水",
        crash="siEarthly",
        soul="jumenMaj",  # 巨门
        body="tianjiMaj",  # 天机
        inside="肾",
        outside="头面",
        health_tip="肾脏、泌尿系统之疾病",
    ),
}


# ============================================================================
# Helper Functions
# ============================================================================


def get_soul_star(earthly_branch: EarthlyBranchName) -> StarName:
    """
    获取地支对应的命主星

    Args:
        earthly_branch: 地支名称

    Returns:
        命主星名称
    """
    return EARTHLY_BRANCHES_CONFIG[earthly_branch].soul


def get_body_star(earthly_branch: EarthlyBranchName) -> StarName:
    """
    获取地支对应的身主星

    Args:
        earthly_branch: 地支名称

    Returns:
        身主星名称
    """
    return EARTHLY_BRANCHES_CONFIG[earthly_branch].body


def get_yin_yang(earthly_branch: EarthlyBranchName) -> YinYang:
    """
    获取地支的阴阳属性

    Args:
        earthly_branch: 地支名称

    Returns:
        阴阳属性
    """
    return EARTHLY_BRANCHES_CONFIG[earthly_branch].yin_yang


def get_five_elements(earthly_branch: EarthlyBranchName) -> FiveElements:
    """
    获取地支的五行属性

    Args:
        earthly_branch: 地支名称

    Returns:
        五行属性
    """
    return EARTHLY_BRANCHES_CONFIG[earthly_branch].five_elements


def get_crash(earthly_branch: EarthlyBranchName) -> EarthlyBranchName:
    """
    获取相冲的地支

    Args:
        earthly_branch: 地支名称

    Returns:
        相冲的地支名称
    """
    return EARTHLY_BRANCHES_CONFIG[earthly_branch].crash


def get_health_info(earthly_branch: EarthlyBranchName) -> Dict[str, str]:
    """
    获取地支相关的健康信息

    Args:
        earthly_branch: 地支名称

    Returns:
        包含 inside, outside, health_tip 的字典
    """
    config = EARTHLY_BRANCHES_CONFIG[earthly_branch]
    return {"inside": config.inside, "outside": config.outside, "health_tip": config.health_tip}



# -*- coding: utf-8 -*-
"""
iztro-py 完整多语言支持示例

展示所有支持的6种语言：
- 简体中文 (zh-CN)
- 繁體中文 (zh-TW)
- English (en-US)
- 日本語 (ja-JP)
- 한국어 (ko-KR)
- Tiếng Việt (vi-VN)
"""

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../src')))

from iztro_py import astro

# 所有支持的语言
LANGUAGES = [
    ('zh-CN', '简体中文', 'Simplified Chinese'),
    ('zh-TW', '繁體中文', 'Traditional Chinese'),
    ('en-US', 'English', 'English'),
    ('ja-JP', '日本語', 'Japanese'),
    ('ko-KR', '한국어', 'Korean'),
    ('vi-VN', 'Tiếng Việt', 'Vietnamese')
]

print("=" * 80)
print("iztro-py 多语言支持演示")
print("Multilingual Support Demo")
print("=" * 80)
print()

# 示例 1: 显示命宫信息（所有语言）
print("示例 1: 命宫信息 (Soul Palace Information)")
print("-" * 80)

for lang_code, lang_native, lang_english in LANGUAGES:
    chart = astro.by_solar('2000-8-16', 6, '男', language=lang_code)
    soul = chart.get_soul_palace()
    
    palace_name = soul.translate_name(lang_code)
    stem = soul.translate_heavenly_stem(lang_code)
    branch = soul.translate_earthly_branch(lang_code)
    
    stars = []
    if soul.major_stars:
        for star in soul.major_stars[:2]:  # 只显示前2颗主星
            star_name = star.translate_name(lang_code)
            brightness = star.translate_brightness(lang_code) or '-'
            stars.append(f"{star_name}({brightness})")
    
    stars_str = ', '.join(stars) if stars else '-'
    
    print(f"{lang_native:15s} ({lang_code}): {palace_name:15s} {stem}{branch}  [{stars_str}]")

print()

# 示例 2: 显示所有宫位名称（选择3种语言对比）
print("示例 2: 十二宫位对比 (12 Palaces Comparison)")
print("-" * 80)

# 选择简体中文、英文、越南语进行对比
compare_langs = [
    ('zh-CN', '简体中文'),
    ('en-US', 'English'),
    ('vi-VN', 'Tiếng Việt')
]

chart = astro.by_solar('2000-8-16', 6, '男')

print(f"{'宫位':4s}  {'简体中文':15s}  {'English':20s}  {'Tiếng Việt':20s}")
print("-" * 80)

for i in range(12):
    palace = chart.palace(i)
    if palace:
        names = []
        for lang_code, _ in compare_langs:
            names.append(palace.translate_name(lang_code))
        
        marker = ""
        if palace.is_original_palace:
            marker = "●命"
        elif palace.is_body_palace:
            marker = "○身"
        
        print(f"{i:2d}{marker:4s}  {names[0]:15s}  {names[1]:20s}  {names[2]:20s}")

print()

# 示例 3: 主星名称对比（14主星）
print("示例 3: 十四主星名称对比 (14 Major Stars Comparison)")
print("-" * 80)

major_star_keys = [
    'ziweiMaj', 'tianjiMaj', 'taiyangMaj', 'wuquMaj',
    'tiantongMaj', 'lianzhenMaj', 'tianfuMaj', 'taiyinMaj',
    'tanlangMaj', 'jumenMaj', 'tianxiangMaj', 'tianliangMaj',
    'qishaMaj', 'pojunMaj'
]

print(f"{'序号':4s}  {'简体中文':10s}  {'繁體中文':10s}  {'日本語':10s}  {'한국어':10s}")
print("-" * 80)

from iztro_py.i18n import t

for idx, star_key in enumerate(major_star_keys, 1):
    zh_cn = t(f'stars.major.{star_key}', 'zh-CN')
    zh_tw = t(f'stars.major.{star_key}', 'zh-TW')
    ja_jp = t(f'stars.major.{star_key}', 'ja-JP')
    ko_kr = t(f'stars.major.{star_key}', 'ko-KR')
    
    print(f"{idx:2d}.   {zh_cn:10s}  {zh_tw:10s}  {ja_jp:10s}  {ko_kr:10s}")

print()

# 示例 4: 天干地支对比
print("示例 4: 天干地支对比 (Heavenly Stems & Earthly Branches)")
print("-" * 80)

heavenly_stems = ['jiaHeavenly', 'yiHeavenly', 'bingHeavenly', 'dingHeavenly', 
                   'wuHeavenly', 'jiHeavenly', 'gengHeavenly', 'xinHeavenly',
                   'renHeavenly', 'guiHeavenly']

earthly_branches = ['ziEarthly', 'chouEarthly', 'yinEarthly', 'maoEarthly',
                     'chenEarthly', 'siEarthly', 'wuEarthly', 'weiEarthly',
                     'shenEarthly', 'youEarthly', 'xuEarthly', 'haiEarthly']

print("天干 (Heavenly Stems):")
stems_zh = [t(f'heavenlyStem.{s}', 'zh-CN') for s in heavenly_stems]
stems_vi = [t(f'heavenlyStem.{s}', 'vi-VN') for s in heavenly_stems]
print(f"  中文: {' '.join(stems_zh)}")
print(f"  Việt: {' '.join(stems_vi)}")

print()
print("地支 (Earthly Branches):")
branches_zh = [t(f'earthlyBranch.{b}', 'zh-CN') for b in earthly_branches]
branches_ko = [t(f'earthlyBranch.{b}', 'ko-KR') for b in earthly_branches]
print(f"  中文: {' '.join(branches_zh)}")
print(f"  한국: {' '.join(branches_ko)}")

print()
print("=" * 80)
print("演示完成！All demos completed!")
print("=" * 80)



"""
iztro-py: A pure Python implementation of iztro

A lightweight library for generating astrolabes for Zi Wei Dou Shu
(紫微斗数, Purple Star Astrology), an ancient Chinese astrology.

This is a pure Python reimplementation of the original JavaScript iztro library,
without any JavaScript interpreter dependencies.
"""

__version__ = "0.3.3"
__author__ = "iztro-py Contributors"
__license__ = "MIT"

# Re-export main modules for convenient imports
from iztro_py import astro, data, star, utils

# Re-export main API functions for easier access
from iztro_py.astro import by_solar, by_lunar

__all__ = ["astro", "data", "star", "utils", "by_solar", "by_lunar", "__version__"]



"""
Core type definitions for iztro-py

This module defines all the type aliases, enums, and data structures used throughout the library.
Based on the original TypeScript definitions from iztro.
"""

from enum import Enum
from typing import Any, Dict, Literal, Optional, List, Tuple, Union
from pydantic import BaseModel, Field, ConfigDict


def _translate_name(key: str, lang: Optional[str] = None) -> str:
    """
    翻译名称的辅助函数
    延迟导入 i18n 模块以避免循环依赖
    """
    from iztro_py.i18n import t

    # 尝试多个可能的键
    # 1. 直接作为星曜名称
    if key in [
        "ziweiMaj",
        "tianjiMaj",
        "taiyangMaj",
        "wuquMaj",
        "tiantongMaj",
        "lianzhenMaj",
        "tianfuMaj",
        "taiyinMaj",
        "tanlangMaj",
        "jumenMaj",
        "tianxiangMaj",
        "tianliangMaj",
        "qishaMaj",
        "pojunMaj",
    ]:
        return t(f"stars.major.{key}", lang)

    if key in [
        "zuofuMin",
        "youbiMin",
        "wenchangMin",
        "wenquMin",
        "tiankuiMin",
        "tianyueMin",
        "huoxingMin",
        "lingxingMin",
        "dikongMin",
        "dijieMin",
        "lucunMin",
        "qingyangMin",
        "tuoluoMin",
        "tianmaMin",
    ]:
        return t(f"stars.minor.{key}", lang)

    # 2. 作为宫位名称
    if key in [
        "soulPalace",
        "parentsPalace",
        "spiritPalace",
        "propertyPalace",
        "careerPalace",
        "friendsPalace",
        "surfacePalace",
        "healthPalace",
        "wealthPalace",
        "childrenPalace",
        "spousePalace",
        "siblingsPalace",
    ]:
        return t(f"palaces.{key}", lang)

    # 3. 作为天干
    if "Heavenly" in key:
        return t(f"heavenlyStem.{key}", lang)

    # 4. 作为地支
    if "Earthly" in key:
        return t(f"earthlyBranch.{key}", lang)

    # 5. 作为时辰
    if "Hour" in key:
        return t(f"time.{key}", lang)

    # 默认返回原值
    return t(key, lang)


# ============================================================================
# Basic Type Aliases
# ============================================================================

Language = Literal["en-US", "ja-JP", "ko-KR", "zh-CN", "zh-TW", "vi-VN"]
YinYang = Literal["阴", "阳"]
FiveElements = Literal["木", "金", "水", "火", "土"]
GenderName = Literal["男", "女"]
Mutagen = Literal["禄", "权", "科", "忌"]
Brightness = Literal["庙", "旺", "得", "利", "平", "不", "陷"]
Scope = Literal["origin", "decadal", "yearly", "monthly", "daily", "hourly"]
StarType = Literal["major", "soft", "tough", "adjective", "flower", "helper", "lucun", "tianma"]


# ============================================================================
# Chinese Time (时辰)
# ============================================================================

ChineseTime = Literal[
    "earlyRatHour",  # 早子时 00:00~01:00
    "oxHour",  # 丑时 01:00~03:00
    "tigerHour",  # 寅时 03:00~05:00
    "rabbitHour",  # 卯时 05:00~07:00
    "dragonHour",  # 辰时 07:00~09:00
    "snakeHour",  # 巳时 09:00~11:00
    "horseHour",  # 午时 11:00~13:00
    "goatHour",  # 未时 13:00~15:00
    "monkeyHour",  # 申时 15:00~17:00
    "roosterHour",  # 酉时 17:00~19:00
    "dogHour",  # 戌时 19:00~21:00
    "pigHour",  # 亥时 21:00~23:00
    "lateRatHour",  # 晚子时 23:00~00:00
]


# ============================================================================
# Heavenly Stems (天干)
# ============================================================================

HeavenlyStemName = Literal[
    "jiaHeavenly",  # 甲
    "yiHeavenly",  # 乙
    "bingHeavenly",  # 丙
    "dingHeavenly",  # 丁
    "wuHeavenly",  # 戊
    "jiHeavenly",  # 己
    "gengHeavenly",  # 庚
    "xinHeavenly",  # 辛
    "renHeavenly",  # 壬
    "guiHeavenly",  # 癸
]


# ============================================================================
# Earthly Branches (地支)
# ============================================================================

EarthlyBranchName = Literal[
    "ziEarthly",  # 子
    "chouEarthly",  # 丑
    "yinEarthly",  # 寅
    "maoEarthly",  # 卯
    "chenEarthly",  # 辰
    "siEarthly",  # 巳
    "wuEarthly",  # 午
    "weiEarthly",  # 未
    "shenEarthly",  # 申
    "youEarthly",  # 酉
    "xuEarthly",  # 戌
    "haiEarthly",  # 亥
]


# ============================================================================
# Palace Names (宫位名称)
# ============================================================================

PalaceName = Literal[
    "soulPalace",  # 命宫
    "parentsPalace",  # 父母宫
    "spiritPalace",  # 福德宫
    "propertyPalace",  # 田宅宫
    "careerPalace",  # 官禄宫
    "friendsPalace",  # 奴仆宫（交友宫）
    "surfacePalace",  # 迁移宫
    "healthPalace",  # 疾厄宫
    "wealthPalace",  # 财帛宫
    "childrenPalace",  # 子女宫
    "spousePalace",  # 夫妻宫
    "siblingsPalace",  # 兄弟宫
]


# ============================================================================
# Star Names (星曜名称)
# ============================================================================

# Major stars (14 main stars)
MajorStarName = Literal[
    "ziweiMaj",  # 紫微
    "tianjiMaj",  # 天机
    "taiyangMaj",  # 太阳
    "wuquMaj",  # 武曲
    "tiantongMaj",  # 天同
    "lianzhenMaj",  # 廉贞
    "tianfuMaj",  # 天府
    "taiyinMaj",  # 太阴
    "tanlangMaj",  # 贪狼
    "jumenMaj",  # 巨门
    "tianxiangMaj",  # 天相
    "tianliangMaj",  # 天梁
    "qishaMaj",  # 七杀
    "pojunMaj",  # 破军
]

# Minor stars (auxiliary stars)
MinorStarName = Literal[
    "zuofuMin",  # 左辅
    "youbiMin",  # 右弼
    "wenchangMin",  # 文昌
    "wenquMin",  # 文曲
    "tiankuiMin",  # 天魁
    "tianyueMin",  # 天钺
    "huoxingMin",  # 火星
    "lingxingMin",  # 铃星
    "dikongMin",  # 地空
    "dijieMin",  # 地劫
    "lucunMin",  # 禄存
    "qingyangMin",  # 擎羊
    "tuoluoMin",  # 陀罗
    "tianmaMin",  # 天马
]

# Adjective stars (杂耀)
AdjectiveStarName = Literal[
    # 长生12神
    "changsheng12",
    # 博士12神
    "boshi12",
    # 流年将前12神
    "jiangqian12",
    # 流年岁前12神
    "suiqian12",
    # 其他杂耀
    "huagaiAdj",  # 华盖
    "xianchiAdj",  # 咸池
    "guchenAdj",  # 孤辰
    "guasuAdj",  # 寡宿
    "tiancaiAdj",  # 天才
    "tianshouAdj",  # 天寿
    "hongluan",  # 红鸾
    "tianxi",  # 天喜
    "tianxing",  # 天刑
    "tianyao",  # 天姚
    "jieshen",  # 解神
    "yinsha",  # 阴煞
    "tianguan",  # 天官
    "tianfu2",  # 天福
    "tianku",  # 天哭
    "tianxu",  # 天虚
    "longchi",  # 龙池
    "fengge",  # 凤阁
    "hongluan",  # 红鸾
    "tianxi",  # 天喜
    "guchen",  # 孤辰
    "guasu",  # 寡宿
    "feilian",  # 蜚廉
    "posui",  # 破碎
    "tianchu",  # 天厨
]

# All star names
StarName = Union[MajorStarName, MinorStarName, AdjectiveStarName]


# ============================================================================
# Five Elements Class (五行局)
# ============================================================================


class FiveElementsClass(Enum):
    """五行局枚举"""

    WATER_2 = 2  # 水二局
    WOOD_3 = 3  # 木三局
    METAL_4 = 4  # 金四局
    EARTH_5 = 5  # 土五局
    FIRE_6 = 6  # 火六局


# ============================================================================
# Data Models
# ============================================================================


class Star(BaseModel):
    """星耀数据结构"""

    name: StarName
    type: StarType
    scope: Scope
    brightness: Optional[Brightness] = None
    mutagen: Optional[Mutagen] = None

    model_config = ConfigDict(frozen=False)  # Allow modification for mutagen/brightness

    def translate_name(self, lang: Optional[str] = None) -> str:
        """
        翻译星曜名称

        Args:
            lang: 目标语言代码，如不指定则使用当前语言

        Returns:
            翻译后的星曜名称
        """
        return _translate_name(self.name, lang)

    def translate_brightness(self, lang: Optional[str] = None) -> Optional[str]:
        """
        翻译亮度

        Args:
            lang: 目标语言代码

        Returns:
            翻译后的亮度，如无亮度则返回 None
        """
        if not self.brightness:
            return None
        from iztro_py.i18n import t

        # 亮度直接就是中文，需要映射到英文键
        brightness_map = {
            "庙": "miao",
            "旺": "wang",
            "得": "de",
            "利": "li",
            "平": "ping",
            "不": "bu",
            "陷": "xian",
        }
        key = brightness_map.get(self.brightness, self.brightness)
        return t(f"brightness.{key}", lang)


class Decadal(BaseModel):
    """大限数据结构"""

    range: Tuple[int, int]  # [起始年龄, 截止年龄]
    heavenly_stem: HeavenlyStemName
    earthly_branch: EarthlyBranchName


class Palace(BaseModel):
    """宫位数据结构"""

    index: int = Field(..., ge=0, le=11)  # 宫位索引 0-11
    name: PalaceName
    is_body_palace: bool = False
    is_original_palace: bool = False
    heavenly_stem: HeavenlyStemName
    earthly_branch: EarthlyBranchName
    major_stars: List[Star] = Field(default_factory=list)
    minor_stars: List[Star] = Field(default_factory=list)
    adjective_stars: List[Star] = Field(default_factory=list)
    changsheng12: Optional[StarName] = None
    boshi12: Optional[StarName] = None
    jiangqian12: Optional[StarName] = None
    suiqian12: Optional[StarName] = None
    decadal: Optional[Decadal] = None
    ages: List[int] = Field(default_factory=list)  # 小限年龄数组

    model_config = ConfigDict(frozen=False)

    def translate_name(self, lang: Optional[str] = None) -> str:
        """
        翻译宫位名称

        Args:
            lang: 目标语言代码

        Returns:
            翻译后的宫位名称
        """
        return _translate_name(self.name, lang)

    def translate_heavenly_stem(self, lang: Optional[str] = None) -> str:
        """翻译天干"""
        return _translate_name(self.heavenly_stem, lang)

    def translate_earthly_branch(self, lang: Optional[str] = None) -> str:
        """翻译地支"""
        return _translate_name(self.earthly_branch, lang)


class SoulAndBody(BaseModel):
    """命身宫定位数据"""

    soul_index: int = Field(..., ge=0, le=11)  # 命宫索引
    body_index: int = Field(..., ge=0, le=11)  # 身宫索引
    heavenly_stem_of_soul: HeavenlyStemName
    earthly_branch_of_soul: EarthlyBranchName


class LunarDate(BaseModel):
    """农历日期"""

    year: int
    month: int
    day: int
    is_leap_month: bool = False


class HeavenlyStemAndEarthlyBranchDate(BaseModel):
    """干支日期（四柱）"""

    year_stem: HeavenlyStemName
    year_branch: EarthlyBranchName
    month_stem: HeavenlyStemName
    month_branch: EarthlyBranchName
    day_stem: HeavenlyStemName
    day_branch: EarthlyBranchName
    time_stem: HeavenlyStemName
    time_branch: EarthlyBranchName


class Astrolabe(BaseModel):
    """星盘数据结构（基础版，不含方法）"""

    gender: GenderName
    solar_date: str  # YYYY-MM-DD
    lunar_date: str  # 农历日期字符串
    chinese_date: str  # 干支日期字符串
    time: str  # 时辰中文名
    time_range: str  # 时间范围 如 '11:00~13:00'
    sign: str  # 星座
    zodiac: str  # 生肖
    earthly_branch_of_soul_palace: EarthlyBranchName
    earthly_branch_of_body_palace: EarthlyBranchName
    soul: StarName  # 命主
    body: StarName  # 身主
    five_elements_class: str  # 五行局
    palaces: List[Palace]

    # Language setting
    language: Language = "zh-CN"

    # Raw dates for internal use
    raw_lunar_date: Optional[LunarDate] = None
    raw_chinese_date: Optional[HeavenlyStemAndEarthlyBranchDate] = None

    model_config = ConfigDict(frozen=False)

    def set_language(self, lang: Language) -> None:
        """
        设置星盘语言

        Args:
            lang: 目标语言代码
        """
        from iztro_py.i18n import set_language

        self.language = lang
        set_language(lang)


class SurroundedPalaces(BaseModel):
    """三方四正宫位（本宫、对宫、财帛、官禄）"""

    target: Palace  # 本宫
    opposite: Palace  # 对宫
    wealth: Palace  # 财帛位（相隔8宫）
    career: Palace  # 官禄位（相隔4宫）

    model_config = ConfigDict(frozen=False)


class HoroscopeItem(BaseModel):
    """运限项"""

    index: int = Field(..., ge=0, le=11)
    name: str
    heavenly_stem: HeavenlyStemName
    earthly_branch: EarthlyBranchName
    palace_names: List[PalaceName]
    mutagen: List[StarName]
    stars: Optional[List[List[Star]]] = None  # 流耀


class Horoscope(BaseModel):
    """运限数据结构"""

    solar_date: str
    lunar_date: str
    decadal: HoroscopeItem  # 大限
    age: HoroscopeItem  # 小限 (包含nominal_age)
    yearly: HoroscopeItem  # 流年
    monthly: HoroscopeItem  # 流月
    daily: HoroscopeItem  # 流日
    hourly: HoroscopeItem  # 流时
    nominal_age: int  # 虚岁


# ============================================================================
# Configuration Types
# ============================================================================


class Config(BaseModel):
    """全局配置"""

    mutagens: Optional[Dict[str, Any]] = None  # 四化配置
    brightness: Optional[Dict[str, Any]] = None  # 亮度配置
    year_divide: Literal["normal", "exact"] = "normal"  # 年分割点
    horoscope_divide: Literal["normal", "exact"] = "normal"  # 运限分割点
    age_divide: Literal["normal", "birthday"] = "normal"  # 小限分割点
    day_divide: Literal["current", "forward"] = "current"  # 晚子时分割
    algorithm: Literal["default", "zhongzhou"] = "default"  # 安星算法


class AstrolabeOptions(BaseModel):
    """星盘生成选项"""

    type: Literal["solar", "lunar"]
    date_str: str
    time_index: int = Field(..., ge=0, le=12)
    gender: GenderName
    is_leap_month: bool = False
    fix_leap: bool = True
    language: Language = "zh-CN"
    config: Optional[Config] = None



"""
FunctionalStar class - Star with functional methods

Provides a rich API for querying star properties and relationships.
"""

from typing import Optional, TYPE_CHECKING, List, Union
from iztro_py.data.types import Star, Brightness, Mutagen

if TYPE_CHECKING:
    from iztro_py.astro.functional_palace import FunctionalPalace
    from iztro_py.astro.functional_surpalaces import FunctionalSurpalaces


class FunctionalStar(Star):
    """
    功能增强的星曜类

    继承自Star，添加了查询方法和关联宫位的能力
    """

    def __init__(self, star: Star):
        """
        初始化FunctionalStar

        Args:
            star: 基础Star对象
        """
        super().__init__(
            name=star.name,
            type=star.type,
            scope=star.scope,
            brightness=star.brightness,
            mutagen=star.mutagen,
        )
        self._palace: Optional["FunctionalPalace"] = None

    def set_palace(self, palace: "FunctionalPalace") -> None:
        """
        设置星曜所在宫位

        Args:
            palace: 宫位对象
        """
        self._palace = palace

    def palace(self) -> Optional["FunctionalPalace"]:
        """
        获取星曜所在宫位

        Returns:
            宫位对象，如果未设置则返回None
        """
        return self._palace

    def with_brightness(self, brightness: Union[Brightness, List[Brightness]]) -> bool:
        """
        判断星曜是否具有指定亮度

        Args:
            brightness: 亮度或亮度列表

        Returns:
            是否匹配

        Example:
            >>> star.with_brightness('庙')
            >>> star.with_brightness(['庙', '旺'])
        """
        if isinstance(brightness, list):
            return self.brightness in brightness
        else:
            return self.brightness == brightness

    def with_mutagen(self, mutagen: Union[Mutagen, List[Mutagen]]) -> bool:
        """
        判断星曜是否具有指定四化

        Args:
            mutagen: 四化类型或四化列表

        Returns:
            是否匹配

        Example:
            >>> star.with_mutagen('禄')
            >>> star.with_mutagen(['禄', '权'])
        """
        if isinstance(mutagen, list):
            return self.mutagen in mutagen
        else:
            return self.mutagen == mutagen

    def opposite_palace(self) -> Optional["FunctionalPalace"]:
        """
        获取星曜的对宫

        Returns:
            对宫对象，如果未设置宫位则返回None
        """
        if not self._palace:
            return None

        from iztro_py.data.constants import get_opposite_index

        opposite_index = get_opposite_index(self._palace.index)

        # 从星盘中获取对宫
        astrolabe = self._palace.astrolabe()
        if astrolabe:
            return astrolabe.palace(opposite_index)

        return None

    def surrounded_palaces(self) -> Optional["FunctionalSurpalaces"]:
        """
        获取星曜的三方四正宫位

        Returns:
            三方四正对象，如果未设置宫位则返回None
        """
        if not self._palace:
            return None

        astrolabe = self._palace.astrolabe()
        if astrolabe:
            return astrolabe.surrounded_palaces(self._palace.index)

        return None

    def is_major(self) -> bool:
        """判断是否为主星"""
        return self.type == "major"

    def is_minor(self) -> bool:
        """判断是否为辅星"""
        return self.type in ["soft", "tough", "lucun", "tianma"]

    def is_bright(self) -> bool:
        """判断是否庙旺（亮度好）"""
        return self.brightness in ["庙", "旺"]

    def is_weak(self) -> bool:
        """判断是否陷弱（亮度差）"""
        return self.brightness == "陷"

    def has_mutagen(self) -> bool:
        """判断是否有四化"""
        return self.mutagen is not None

    def __str__(self) -> str:
        """字符串表示"""
        parts = [self.name]
        if self.brightness:
            parts.append(f"({self.brightness})")
        if self.mutagen:
            parts.append(f"[化{self.mutagen}]")
        return "".join(parts)

    def __repr__(self) -> str:
        return f"FunctionalStar({self.name}, brightness={self.brightness}, mutagen={self.mutagen})"



"""
Astro module: Core astrolabe functionality and palace calculations.
"""

from iztro_py.astro.astro import (
    by_solar,
    by_lunar,
    by_solar_hour,
    by_lunar_hour,
    get_zodiac_by_solar_date,
    get_sign_by_solar_date,
)
from iztro_py.astro.functional_astrolabe import FunctionalAstrolabe
from iztro_py.astro.functional_palace import FunctionalPalace
from iztro_py.astro.functional_star import FunctionalStar
from iztro_py.astro.functional_surpalaces import FunctionalSurpalaces

__all__ = [
    "by_solar",
    "by_lunar",
    "by_solar_hour",
    "by_lunar_hour",
    "get_zodiac_by_solar_date",
    "get_sign_by_solar_date",
    "FunctionalAstrolabe",
    "FunctionalPalace",
    "FunctionalStar",
    "FunctionalSurpalaces",
]



"""
FunctionalAstrolabe class - Complete astrolabe with functional methods

The main class for interacting with a Zi Wei Dou Shu astrolabe.
Provides rich API for querying palaces, stars, and their relationships.
"""

from typing import List, Optional, Union
from iztro_py.data.types import Astrolabe, PalaceName, StarName
from iztro_py.astro.functional_palace import FunctionalPalace
from iztro_py.astro.functional_star import FunctionalStar
from iztro_py.astro.functional_surpalaces import FunctionalSurpalaces
from iztro_py.data.constants import get_surrounded_indices
from iztro_py.i18n import t


class FunctionalAstrolabe(Astrolabe):
    """
    功能增强的星盘类

    继承自Astrolabe，提供丰富的查询和链式调用API
    """

    def __init__(self, astrolabe: Astrolabe):
        """
        初始化FunctionalAstrolabe

        Args:
            astrolabe: 基础Astrolabe对象
        """
        # 转换宫位为FunctionalPalace
        functional_palaces = [FunctionalPalace(p) for p in astrolabe.palaces]

        super().__init__(
            gender=astrolabe.gender,
            solar_date=astrolabe.solar_date,
            lunar_date=astrolabe.lunar_date,
            chinese_date=astrolabe.chinese_date,
            time=astrolabe.time,
            time_range=astrolabe.time_range,
            sign=astrolabe.sign,
            zodiac=astrolabe.zodiac,
            earthly_branch_of_soul_palace=astrolabe.earthly_branch_of_soul_palace,
            earthly_branch_of_body_palace=astrolabe.earthly_branch_of_body_palace,
            soul=astrolabe.soul,
            body=astrolabe.body,
            five_elements_class=astrolabe.five_elements_class,
            palaces=functional_palaces,
            raw_lunar_date=astrolabe.raw_lunar_date,
            raw_chinese_date=astrolabe.raw_chinese_date,
        )

        # 设置宫位的星盘引用
        for palace in self.palaces:
            palace.set_astrolabe(self)

    def palace(self, index_or_name: Union[int, PalaceName]) -> Optional[FunctionalPalace]:
        """
        获取指定的宫位对象

        Args:
            index_or_name: 宫位索引 (0-11) 或宫位名称

        Returns:
            宫位对象，如果不存在则返回None

        Example:
            >>> astrolabe.palace(0)
            >>> astrolabe.palace('soulPalace')
            >>> astrolabe.palace('命宫')
        """
        if isinstance(index_or_name, int):
            # 按索引查询
            if 0 <= index_or_name < len(self.palaces):
                return self.palaces[index_or_name]
            return None
        else:
            # 按名称查询
            # 先尝试英文名
            for palace in self.palaces:
                if palace.name == index_or_name:
                    return palace

            # 再尝试中文名
            from iztro_py.utils.helpers import get_palace_index_by_name

            palace_index = get_palace_index_by_name(index_or_name)
            if palace_index is not None:
                return self.palaces[palace_index]

            return None

    def star(self, star_name: StarName) -> Optional[FunctionalStar]:
        """
        获取指定的星曜对象

        Args:
            star_name: 星曜名称

        Returns:
            星曜对象，如果不存在则返回None

        Example:
            >>> astrolabe.star('ziweiMaj')
            >>> astrolabe.star('紫微')
        """
        for palace in self.palaces:
            star = palace.get_star(star_name)
            if star:
                return star

        return None

    def surrounded_palaces(
        self, index_or_name: Union[int, PalaceName]
    ) -> Optional[FunctionalSurpalaces]:
        """
        获取指定宫位的三方四正

        Args:
            index_or_name: 宫位索引或名称

        Returns:
            三方四正对象，如果宫位不存在则返回None

        Example:
            >>> astrolabe.surrounded_palaces(0)
            >>> astrolabe.surrounded_palaces('命宫')
        """
        target_palace = self.palace(index_or_name)
        if not target_palace:
            return None

        # 获取三方四正的索引
        indices = get_surrounded_indices(target_palace.index)

        opposite_palace = self.palaces[indices["opposite"]]
        wealth_palace = self.palaces[indices["wealth"]]
        career_palace = self.palaces[indices["career"]]

        return FunctionalSurpalaces(
            target=target_palace,
            opposite=opposite_palace,
            wealth=wealth_palace,
            career=career_palace,
        )

    def not_empty_palaces(self) -> List[FunctionalPalace]:
        """
        获取所有非空宫（有主星的宫位）

        Returns:
            非空宫列表
        """
        return [p for p in self.palaces if not p.is_empty()]

    def empty_palaces(self) -> List[FunctionalPalace]:
        """
        获取所有空宫（无主星的宫位）

        Returns:
            空宫列表
        """
        return [p for p in self.palaces if p.is_empty()]

    def get_soul_palace(self) -> Optional[FunctionalPalace]:
        """
        获取命宫

        Returns:
            命宫对象
        """
        for palace in self.palaces:
            if palace.is_original_palace:
                return palace
        return None

    def get_body_palace(self) -> Optional[FunctionalPalace]:
        """
        获取身宫

        Returns:
            身宫对象
        """
        for palace in self.palaces:
            if palace.is_body_palace:
                return palace
        return None

    def horoscope(self, solar_date: str, time_index: int = 0):
        """
        获取指定日期的运势信息（大限、流年、流月、流日、流时）

        Args:
            solar_date: 查询的阳历日期 (YYYY-M-D or YYYY-MM-DD)
            time_index: 时辰索引 (0-12)，默认为0（子时）

        Returns:
            Horoscope对象，包含大限、流年、流月、流日、流时信息

        Example:
            >>> chart = astro.by_solar('2000-8-16', 6, '男')
            >>> horoscope = chart.horoscope('2024-1-1', 6)
            >>> print(f"大限: {horoscope.decadal.name}")
            >>> print(f"流年: {horoscope.yearly.name}")
        """
        from iztro_py.astro.horoscope import get_horoscope
        from iztro_py.data.types import FiveElementsClass

        # 获取出生年份
        birth_year = int(self.solar_date.split("-")[0])

        # 获取命宫索引
        soul_palace = self.get_soul_palace()
        soul_palace_index = soul_palace.index if soul_palace else 0

        # 获取五行局
        five_elements_class_map = {
            "水二局": FiveElementsClass.WATER_2,
            "木三局": FiveElementsClass.WOOD_3,
            "金四局": FiveElementsClass.METAL_4,
            "土五局": FiveElementsClass.EARTH_5,
            "火六局": FiveElementsClass.FIRE_6,
        }
        five_elements = five_elements_class_map.get(
            self.five_elements_class, FiveElementsClass.WATER_2
        )

        # 获取出生年支阴阳
        if self.raw_chinese_date:
            year_branch = self.raw_chinese_date.year_branch
            # 从地支获取阴阳
            from iztro_py.data.earthly_branches import EARTHLY_BRANCHES_CONFIG

            branch_config = EARTHLY_BRANCHES_CONFIG.get(year_branch)
            year_branch_yin_yang = branch_config.yin_yang if branch_config else "阳"
        else:
            year_branch_yin_yang = "阳"

        return get_horoscope(
            solar_date_str=solar_date,
            time_index=time_index,
            palaces=self.palaces,
            soul_palace_index=soul_palace_index,
            five_elements_class=five_elements,
            gender=self.gender,
            year_branch_yin_yang=year_branch_yin_yang,
            birth_year=birth_year,
        )

    def __str__(self) -> str:
        """字符串表示"""
        lines = [
            f"紫微斗数星盘",
            f"出生日期: {self.solar_date} ({self.lunar_date})",
            f"性别: {self.gender}",
            f"生肖: {self.zodiac} | 星座: {self.sign}",
            f"五行局: {self.five_elements_class}",
            f"命主: {self.soul} | 身主: {self.body}",
            "",
            "十二宫:",
        ]

        for palace in self.palaces:
            lines.append(f"  {palace}")

        return "\n".join(lines)

    def __repr__(self) -> str:
        return f"FunctionalAstrolabe(date={self.solar_date}, gender={self.gender})"

    # ---------------------------------------------------------------------
    # Compatibility Export
    # ---------------------------------------------------------------------
    def to_iztro_dict(self) -> dict:
        """
        导出与原生 iztro/py-iztro 结构一致的字典（字段名与中文值对齐）

        返回字段示例：
        - gender, solarDate, lunarDate, chineseDate, time, timeRange, sign, zodiac
        - earthlyBranchOfSoulPalace, earthlyBranchOfBodyPalace, soul, body, fiveElementsClass
        - palaces: [{ name, isBodyPalace, isOriginalPalace, heavenlyStem, earthlyBranch, majorStars, minorStars }]
        """

        def tr_branch(branch_key: str) -> str:
            return t(f"earthlyBranch.{branch_key}") if "Earthly" in branch_key else branch_key

        def tr_stem(stem_key: str) -> str:
            return t(f"heavenlyStem.{stem_key}") if "Heavenly" in stem_key else stem_key

        def star_dict(star: FunctionalStar) -> dict:
            return {
                "name": star.translate_name(),
                "type": star.type,
                "scope": star.scope,
                "brightness": star.brightness,
                "mutagen": star.mutagen,
            }

        palaces = []
        for p in self.palaces:
            palaces.append(
                {
                    "name": p.translate_name(),
                    "isBodyPalace": p.is_body_palace,
                    "isOriginalPalace": p.is_original_palace,
                    "heavenlyStem": tr_stem(p.heavenly_stem),
                    "earthlyBranch": tr_branch(p.earthly_branch),
                    "majorStars": [star_dict(s) for s in p.major_stars],
                    "minorStars": [star_dict(s) for s in p.minor_stars],
                    "adjectiveStars": [star_dict(s) for s in p.adjective_stars],
                }
            )

        from iztro_py.data.types import Star

        return {
            "gender": self.gender,
            "solarDate": self.solar_date,
            "lunarDate": self.lunar_date,
            "chineseDate": self.chinese_date,
            "time": self.time,
            "timeRange": self.time_range,
            "sign": self.sign,
            "zodiac": self.zodiac,
            "earthlyBranchOfSoulPalace": tr_branch(self.earthly_branch_of_soul_palace),
            "earthlyBranchOfBodyPalace": tr_branch(self.earthly_branch_of_body_palace),
            "soul": Star(name=self.soul, type="major", scope="origin").translate_name(),
            "body": Star(name=self.body, type="major", scope="origin").translate_name(),
            "fiveElementsClass": self.five_elements_class,
            "palaces": palaces,
        }



"""
Minor stars placement for iztro-py

Functions for placing the 14 minor stars (辅星) into palaces.
"""

from typing import Any, Dict, List, Optional
from iztro_py.data.types import Star, HeavenlyStemName, EarthlyBranchName
from iztro_py.data.constants import HEAVENLY_STEMS, EARTHLY_BRANCHES
from iztro_py.star.location import (
    get_minor_star_position_zuofu,
    get_minor_star_position_youbi,
    get_minor_star_position_wenchang,
    get_minor_star_position_wenqu,
    get_minor_star_positions_kuiyue,
    get_minor_star_positions_huoling,
    get_minor_star_positions_kongjie,
    get_minor_star_positions_lucun_yangtuo_tianma,
)


def _find_palace_by_branch_index(
    palaces: List[Dict[str, Any]], branch_index: int
) -> Optional[Dict[str, Any]]:
    """
    根据地支索引查找对应的宫位

    Args:
        palaces: 宫位列表
        branch_index: 地支索引 (0-11)

    Returns:
        对应的宫位字典，如果未找到则返回None
    """
    target_branch = EARTHLY_BRANCHES[branch_index]
    for palace in palaces:
        if palace["earthly_branch"] == target_branch:
            return palace
    return None


def place_minor_stars(
    palaces: List[Dict[str, Any]],
    lunar_month: int,
    time_index: int,
    year_stem: HeavenlyStemName,
    year_branch: EarthlyBranchName,
) -> None:
    """
    将14颗辅星安置到宫位中

    Args:
        palaces: 宫位列表
        lunar_month: 农历月
        time_index: 时辰索引
        year_stem: 年干
        year_branch: 年支

    Note:
        直接修改palaces列表，不返回值
    """
    year_stem_index = HEAVENLY_STEMS.index(year_stem)
    year_branch_index = EARTHLY_BRANCHES.index(year_branch)

    # 1. 左辅、右弼（按农历月）
    zuofu_index = get_minor_star_position_zuofu(lunar_month)
    youbi_index = get_minor_star_position_youbi(lunar_month)

    zuofu_palace = _find_palace_by_branch_index(palaces, zuofu_index)
    if zuofu_palace:
        zuofu_palace["minor_stars"].append(Star(name="zuofuMin", type="soft", scope="origin"))

    youbi_palace = _find_palace_by_branch_index(palaces, youbi_index)
    if youbi_palace:
        youbi_palace["minor_stars"].append(Star(name="youbiMin", type="soft", scope="origin"))

    # 2. 文昌、文曲（按时辰）
    wenchang_index = get_minor_star_position_wenchang(time_index)
    wenqu_index = get_minor_star_position_wenqu(time_index)

    wenchang_palace = _find_palace_by_branch_index(palaces, wenchang_index)
    if wenchang_palace:
        wenchang_palace["minor_stars"].append(Star(name="wenchangMin", type="soft", scope="origin"))

    wenqu_palace = _find_palace_by_branch_index(palaces, wenqu_index)
    if wenqu_palace:
        wenqu_palace["minor_stars"].append(Star(name="wenquMin", type="soft", scope="origin"))

    # 3. 天魁、天钺（按年干）
    kuai_index, yue_index = get_minor_star_positions_kuiyue(year_stem_index)

    kuai_palace = _find_palace_by_branch_index(palaces, kuai_index)
    if kuai_palace:
        kuai_palace["minor_stars"].append(Star(name="tiankuiMin", type="soft", scope="origin"))

    yue_palace = _find_palace_by_branch_index(palaces, yue_index)
    if yue_palace:
        yue_palace["minor_stars"].append(Star(name="tianyueMin", type="soft", scope="origin"))

    # 4. 火星、铃星（按年支和时辰）
    huo_index, ling_index = get_minor_star_positions_huoling(year_branch_index, time_index)

    huo_palace = _find_palace_by_branch_index(palaces, huo_index)
    if huo_palace:
        huo_palace["minor_stars"].append(Star(name="huoxingMin", type="tough", scope="origin"))

    ling_palace = _find_palace_by_branch_index(palaces, ling_index)
    if ling_palace:
        ling_palace["minor_stars"].append(Star(name="lingxingMin", type="tough", scope="origin"))

    # 5. 地空、地劫（按时辰）
    kong_index, jie_index = get_minor_star_positions_kongjie(time_index)

    kong_palace = _find_palace_by_branch_index(palaces, kong_index)
    if kong_palace:
        kong_palace["minor_stars"].append(Star(name="dikongMin", type="tough", scope="origin"))

    jie_palace = _find_palace_by_branch_index(palaces, jie_index)
    if jie_palace:
        jie_palace["minor_stars"].append(Star(name="dijieMin", type="tough", scope="origin"))

    # 6. 禄存、擎羊、陀罗、天马（按年干支）
    lucun_index, yang_index, tuo_index, tianma_index = (
        get_minor_star_positions_lucun_yangtuo_tianma(year_stem_index, year_branch_index)
    )

    lucun_palace = _find_palace_by_branch_index(palaces, lucun_index)
    if lucun_palace:
        lucun_palace["minor_stars"].append(Star(name="lucunMin", type="lucun", scope="origin"))

    yang_palace = _find_palace_by_branch_index(palaces, yang_index)
    if yang_palace:
        yang_palace["minor_stars"].append(Star(name="qingyangMin", type="tough", scope="origin"))

    tuo_palace = _find_palace_by_branch_index(palaces, tuo_index)
    if tuo_palace:
        tuo_palace["minor_stars"].append(Star(name="tuoluoMin", type="tough", scope="origin"))

    tianma_palace = _find_palace_by_branch_index(palaces, tianma_index)
    if tianma_palace:
        tianma_palace["minor_stars"].append(Star(name="tianmaMin", type="tianma", scope="origin"))


def get_minor_stars_in_palace(palace: dict) -> List[Star]:
    """
    获取指定宫位中的辅星列表

    Args:
        palace: 宫位字典

    Returns:
        辅星列表
    """
    return palace.get("minor_stars", [])


def has_minor_star(palace: dict, star_name: str) -> bool:
    """
    判断宫位是否包含指定辅星

    Args:
        palace: 宫位字典
        star_name: 星曜名称

    Returns:
        是否包含
    """
    minor_stars = get_minor_stars_in_palace(palace)
    return any(star.name == star_name for star in minor_stars)



"""
iztro-py 运势系统使用示例

演示如何使用运势系统查询大限、流年、流月、流日、流时等信息。
"""

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../src')))

from iztro_py import astro

# ============================================================
# 示例 1：查询完整运势信息
# ============================================================
print("=" * 60)
print("示例 1：查询完整运势信息")
print("=" * 60)

# 创建星盘：2000年8月16日午时出生的男命
chart = astro.by_solar('2000-8-16', 6, '男')

print(f"出生日期: {chart.solar_date}")
print(f"性别: {chart.gender}")
print(f"生肖: {chart.zodiac}")
print(f"五行局: {chart.five_elements_class}")
print()

# 查询2024年1月1日午时的运势
horoscope = chart.horoscope('2024-1-1', 6)

print(f"查询日期: {horoscope.solar_date} ({horoscope.lunar_date})")
print(f"虚岁: {horoscope.nominal_age}")
print()

# 大限（10年一运）
print(f"【大限】{horoscope.decadal.name}")
print(f"  所在宫位: {', '.join(horoscope.decadal.palace_names)}")
print(f"  天干地支: {horoscope.decadal.heavenly_stem} {horoscope.decadal.earthly_branch}")
decadal_palace = chart.palace(horoscope.decadal.index)
if decadal_palace:
    stars = ', '.join(s.name for s in decadal_palace.major_stars) or '空宫'
    print(f"  宫内主星: {stars}")
print()

# 小限（1年一运）
print(f"【小限】{horoscope.age.name}")
print(f"  所在宫位: {', '.join(horoscope.age.palace_names)}")
age_palace = chart.palace(horoscope.age.index)
if age_palace:
    stars = ', '.join(s.name for s in age_palace.major_stars) or '空宫'
    print(f"  宫内主星: {stars}")
print()

# 流年
print(f"【流年】{horoscope.yearly.name}")
print(f"  流年命宫在本命: {', '.join(horoscope.yearly.palace_names)}")
yearly_palace = chart.palace(horoscope.yearly.index)
if yearly_palace:
    stars = ', '.join(s.name for s in yearly_palace.major_stars) or '空宫'
    print(f"  宫内主星: {stars}")
    print(f"  流年四化: {', '.join(horoscope.yearly.mutagen[:4])}")
print()

# 流月
print(f"【流月】{horoscope.monthly.name}")
print(f"  流月命宫在本命: {', '.join(horoscope.monthly.palace_names)}")
print()

# 流日
print(f"【流日】{horoscope.daily.name}")
print(f"  流日命宫在本命: {', '.join(horoscope.daily.palace_names)}")
print()

# 流时
print(f"【流时】{horoscope.hourly.name}")
print(f"  流时命宫在本命: {', '.join(horoscope.hourly.palace_names)}")
print()


# ============================================================
# 示例 2：追踪一年中的运势变化
# ============================================================
print("=" * 60)
print("示例 2：追踪一年中的运势变化")
print("=" * 60)

chart = astro.by_solar('1990-1-1', 0, '女')

# 查看2024年1、4、7、10月的运势
months = ['2024-1-1', '2024-4-1', '2024-7-1', '2024-10-1']

for date_str in months:
    h = chart.horoscope(date_str, 0)
    print(f"{date_str}")
    print(f"  流月: {h.monthly.name}, 宫位: {h.monthly.palace_names[0]}")
    monthly_palace = chart.palace(h.monthly.index)
    if monthly_palace:
        stars = ', '.join(s.name for s in monthly_palace.major_stars) or '空'
        print(f"  主星: {stars}")
    print()


# ============================================================
# 示例 3：大限分析（每10年的运程）
# ============================================================
print("=" * 60)
print("示例 3：大限分析（每10年的运程）")
print("=" * 60)

chart = astro.by_solar('2000-8-16', 6, '男')

print(f"出生: {chart.solar_date}, {chart.gender}, {chart.five_elements_class}")
print()

# 分析从10岁到50岁的大限
ages = [10, 20, 30, 40, 50]

for age in ages:
    # 假设查询该年龄的1月1日
    birth_year = 2000
    query_year = birth_year + age - 1  # 虚岁
    h = chart.horoscope(f'{query_year}-1-1', 6)

    print(f"{age}岁 ({query_year}年)")
    print(f"  大限: {h.decadal.name}")
    decadal_palace = chart.palace(h.decadal.index)
    if decadal_palace:
        print(f"  宫位: {decadal_palace.name}")
        stars = ', '.join(s.name for s in decadal_palace.major_stars) or '空宫'
        print(f"  主星: {stars}")
    print()


# ============================================================
# 示例 4：结合三方四正分析流年
# ============================================================
print("=" * 60)
print("示例 4：结合三方四正分析流年")
print("=" * 60)

chart = astro.by_solar('2000-8-16', 6, '男')
horoscope = chart.horoscope('2024-6-15', 6)

print(f"查询日期: 2024-6-15")
print(f"流年: {horoscope.yearly.name}")
print()

# 获取流年命宫
yearly_palace = chart.palace(horoscope.yearly.index)
if yearly_palace:
    print(f"流年命宫在本命: {yearly_palace.name}")
    stars = ', '.join(s.name for s in yearly_palace.major_stars) or '空'
    print(f"  主星: {stars}")
    print()

    # 查看流年命宫的三方四正
    surpalaces = chart.surrounded_palaces(horoscope.yearly.index)
    if surpalaces:
        print("流年命宫三方四正:")
        for p in surpalaces.all_palaces():
            stars_list = ', '.join(s.name for s in p.major_stars) or '空'
            # 显示四化星
            mutagen_stars = [f"{s.name}化{s.mutagen}" for s in p.major_stars if s.mutagen]
            mutagen_str = f" ({', '.join(mutagen_stars)})" if mutagen_stars else ""
            print(f"  {p.name:15s}: {stars_list}{mutagen_str}")
        print()

        # 分析三方四正的吉凶
        print("三方四正分析:")
        has_lu = surpalaces.have_mutagen('禄')
        has_ji = surpalaces.have_mutagen('忌')
        print(f"  有化禄: {'是' if has_lu else '否'}")
        print(f"  有化忌: {'是' if has_ji else '否'}")
        print()


# ============================================================
# 示例 5：男女命大限对比
# ============================================================
print("=" * 60)
print("示例 5：男女命大限对比（同日同时）")
print("=" * 60)

# 同一天同一时辰出生的男女命
chart_male = astro.by_solar('2000-8-16', 6, '男')
chart_female = astro.by_solar('2000-8-16', 6, '女')

query_date = '2024-1-1'
h_male = chart_male.horoscope(query_date, 6)
h_female = chart_female.horoscope(query_date, 6)

print(f"出生: 2000-8-16 午时")
print(f"查询: {query_date}")
print()

print("男命:")
print(f"  大限: {h_male.decadal.name}, 宫位: {h_male.decadal.palace_names[0]}")
decadal_male = chart_male.palace(h_male.decadal.index)
if decadal_male:
    stars = ', '.join(s.name for s in decadal_male.major_stars) or '空'
    print(f"  主星: {stars}")
print()

print("女命:")
print(f"  大限: {h_female.decadal.name}, 宫位: {h_female.decadal.palace_names[0]}")
decadal_female = chart_female.palace(h_female.decadal.index)
if decadal_female:
    stars = ', '.join(s.name for s in decadal_female.major_stars) or '空'
    print(f"  主星: {stars}")
print()

print("注: 男女命大限顺逆行不同，因此相同年龄的大限宫位也不同")
print()


print("=" * 60)
print("示例完成！")
print("=" * 60)



"""
Helper utilities for iztro-py

Common utility functions used throughout the library.
"""

from typing import Optional, Tuple
from iztro_py.data.types import HeavenlyStemName, EarthlyBranchName, FiveElementsClass
from iztro_py.data.constants import (
    HEAVENLY_STEMS,
    EARTHLY_BRANCHES,
    FIVE_ELEMENTS_CLASS_LOOKUP,
    fix_index,
)


def get_five_elements_class(
    heavenly_stem: HeavenlyStemName, earthly_branch: EarthlyBranchName
) -> FiveElementsClass:
    """
    根据命宫天干地支获取五行局

    Args:
        heavenly_stem: 命宫天干
        earthly_branch: 命宫地支

    Returns:
        五行局枚举值
    """
    stem_index = HEAVENLY_STEMS.index(heavenly_stem)
    branch_index = EARTHLY_BRANCHES.index(earthly_branch)

    class_value = FIVE_ELEMENTS_CLASS_LOOKUP[stem_index][branch_index]

    # 转换为枚举
    if class_value == 2:
        return FiveElementsClass.WATER_2
    elif class_value == 3:
        return FiveElementsClass.WOOD_3
    elif class_value == 4:
        return FiveElementsClass.METAL_4
    elif class_value == 5:
        return FiveElementsClass.EARTH_5
    elif class_value == 6:
        return FiveElementsClass.FIRE_6
    else:
        raise ValueError(f"Invalid five elements class value: {class_value}")


def get_five_elements_class_name(five_elements_class: FiveElementsClass) -> str:
    """
    获取五行局的中文名称

    Args:
        five_elements_class: 五行局枚举

    Returns:
        中文名称，如 "水二局"
    """
    names = {
        FiveElementsClass.WATER_2: "水二局",
        FiveElementsClass.WOOD_3: "木三局",
        FiveElementsClass.METAL_4: "金四局",
        FiveElementsClass.EARTH_5: "土五局",
        FiveElementsClass.FIRE_6: "火六局",
    }
    return names[five_elements_class]


def get_time_range(time_index: int) -> str:
    """
    获取时辰的时间范围

    Args:
        time_index: 时辰索引 (0-12)

    Returns:
        时间范围字符串，如 "11:00~13:00"
    """
    time_ranges = [
        "00:00~01:00",  # 早子时
        "01:00~03:00",  # 丑时
        "03:00~05:00",  # 寅时
        "05:00~07:00",  # 卯时
        "07:00~09:00",  # 辰时
        "09:00~11:00",  # 巳时
        "11:00~13:00",  # 午时
        "13:00~15:00",  # 未时
        "15:00~17:00",  # 申时
        "17:00~19:00",  # 酉时
        "19:00~21:00",  # 戌时
        "21:00~23:00",  # 亥时
        "23:00~00:00",  # 晚子时
    ]

    if 0 <= time_index < len(time_ranges):
        return time_ranges[time_index]
    else:
        raise ValueError(f"Invalid time index: {time_index}. Must be 0-12.")


def get_time_name(time_index: int) -> str:
    """
    获取时辰的中文名称

    Args:
        time_index: 时辰索引 (0-12)

    Returns:
        时辰中文名称，如 "午时"
    """
    time_names = [
        "子时",  # 0 早子时
        "丑时",  # 1
        "寅时",  # 2
        "卯时",  # 3
        "辰时",  # 4
        "巳时",  # 5
        "午时",  # 6
        "未时",  # 7
        "申时",  # 8
        "酉时",  # 9
        "戌时",  # 10
        "亥时",  # 11
        "子时",  # 12 晚子时
    ]

    if 0 <= time_index < len(time_names):
        return time_names[time_index]
    else:
        raise ValueError(f"Invalid time index: {time_index}. Must be 0-12.")


def hour_to_time_index(hour: int) -> int:
    """
    将 24 小时制的小时数 (0-23) 映射为时辰索引 (0-12)

    规则与 iztro 保持一致：
    - 子时分早子时(0)与晚子时(12)：23点对应晚子时(12)，0点对应早子时(0)
    - 其他小时按每两小时一个时辰：(hour + 1) // 2

    Args:
        hour: 小时 (0-23)

    Returns:
        时辰索引 (0-12)
    """
    if not (0 <= hour <= 23):
        raise ValueError(f"hour must be in 0..23, got {hour}")

    if hour == 23:
        return 12  # 晚子时
    return (hour + 1) // 2


def calculate_nominal_age(birth_year: int, target_year: int, age_divide: str = "normal") -> int:
    """
    计算虚岁

    Args:
        birth_year: 出生年份
        target_year: 目标年份
        age_divide: 年龄划分方式
            - 'normal': 按自然年计算
            - 'birthday': 按生日计算（需要完整日期，此处简化处理）

    Returns:
        虚岁
    """
    # 虚岁 = 当前年份 - 出生年份 + 1
    return target_year - birth_year + 1


def get_palace_index_by_name(palace_name: str) -> Optional[int]:
    """
    根据宫位名称获取索引

    Args:
        palace_name: 宫位名称（中文或英文key）

    Returns:
        宫位索引 (0-11)，如果未找到返回None
    """
    # 中文名称映射
    chinese_names = {
        "命宫": 0,
        "父母宫": 1,
        "福德宫": 2,
        "田宅宫": 3,
        "官禄宫": 4,
        "奴仆宫": 5,
        "交友宫": 5,  # 奴仆宫别名
        "迁移宫": 6,
        "疾厄宫": 7,
        "财帛宫": 8,
        "子女宫": 9,
        "夫妻宫": 10,
        "兄弟宫": 11,
        # 简化别名（不带"宫"）
        "命": 0,
        "父母": 1,
        "福德": 2,
        "田宅": 3,
        "官禄": 4,
        "事业": 4,  # 官禄宫别名
        "奴仆": 5,
        "交友": 5,
        "迁移": 6,
        "疾厄": 7,
        "财帛": 8,
        "子女": 9,
        "夫妻": 10,
        "兄弟": 11,
    }

    # 英文key映射
    english_keys = {
        "soulPalace": 0,
        "parentsPalace": 1,
        "spiritPalace": 2,
        "propertyPalace": 3,
        "careerPalace": 4,
        "friendsPalace": 5,
        "surfacePalace": 6,
        "healthPalace": 7,
        "wealthPalace": 8,
        "childrenPalace": 9,
        "spousePalace": 10,
        "siblingsPalace": 11,
        # 简化别名（不带"Palace"）
        "soul": 0,
        "parents": 1,
        "spirit": 2,
        "property": 3,
        "career": 4,
        "friends": 5,
        "surface": 6,
        "health": 7,
        "wealth": 8,
        "children": 9,
        "spouse": 10,
        "siblings": 11,
    }

    # 先尝试中文
    if palace_name in chinese_names:
        return chinese_names[palace_name]

    # 再尝试英文
    if palace_name in english_keys:
        return english_keys[palace_name]

    return None


def get_decadal_range(
    five_elements_class: FiveElementsClass,
    palace_index: int,
    gender: str,
    soul_palace_index: int = 0,
    year_branch_yin_yang: str = "阳",
) -> Tuple[int, int]:
    """
    计算大限年龄范围

    大限从命宫开始，每个宫位管10年
    男命阳年生人、女命阴年生人：顺行
    男命阴年生人、女命阳年生人：逆行

    Args:
        five_elements_class: 五行局
        palace_index: 宫位索引 (0-11)
        gender: 性别 ('男' 或 '女')
        soul_palace_index: 命宫索引（默认0）
        year_branch_yin_yang: 年支阴阳（默认'阳'）

    Returns:
        (起始年龄, 截止年龄) 元组
    """
    # 五行局决定起始年龄
    start_age = five_elements_class.value

    # 判断顺逆行
    # 男阳女阴顺行，男阴女阳逆行
    is_forward = (gender == "男" and year_branch_yin_yang == "阳") or (
        gender == "女" and year_branch_yin_yang == "阴"
    )

    # 计算此宫位是第几个大限（从0开始）
    if is_forward:
        # 顺行：从命宫开始往后数
        decadal_order = (palace_index - soul_palace_index) % 12
    else:
        # 逆行：从命宫开始往前数
        decadal_order = (soul_palace_index - palace_index) % 12

    # 起始年龄
    range_start = start_age + decadal_order * 10
    range_end = range_start + 9

    return (range_start, range_end)


def get_decadal_palace_index(
    age: int,
    five_elements_class: FiveElementsClass,
    soul_palace_index: int,
    gender: str,
    year_branch_yin_yang: str = "阳",
) -> int:
    """
    根据年龄获取大限所在的宫位索引

    Args:
        age: 虚岁
        five_elements_class: 五行局
        soul_palace_index: 命宫索引
        gender: 性别
        year_branch_yin_yang: 年支阴阳

    Returns:
        大限宫位索引
    """
    start_age = five_elements_class.value

    # 判断顺逆行
    # 男阳女阴顺行，男阴女阳逆行
    is_forward = (gender == "男" and year_branch_yin_yang == "阳") or (
        gender == "女" and year_branch_yin_yang == "阴"
    )

    # 计算从命宫开始的第几个大限（从0开始）
    decadal_order = (age - start_age) // 10

    if is_forward:
        # 顺行
        palace_index = fix_index(soul_palace_index + decadal_order)
    else:
        # 逆行
        palace_index = fix_index(soul_palace_index - decadal_order)

    return palace_index



"""
Utils module: Utility functions and helper methods.
"""

from iztro_py.utils import calendar, helpers

from iztro_py.utils.calendar import (
    solar_to_lunar,
    lunar_to_solar,
    parse_solar_date,
    parse_lunar_date,
    get_year_stem_branch,
    get_month_stem_branch,
    get_day_stem_branch,
    get_time_stem_branch,
    get_heavenly_stem_and_earthly_branch_date,
    get_zodiac,
    get_sign,
    format_lunar_date,
    format_chinese_date,
    ZODIAC_NAMES,
)

from iztro_py.utils.helpers import (
    get_five_elements_class,
    get_five_elements_class_name,
    get_time_range,
    get_time_name,
    calculate_nominal_age,
    get_palace_index_by_name,
    get_decadal_range,
    get_decadal_palace_index,
)

__all__ = [
    # Submodules
    "calendar",
    "helpers",
    # Calendar functions
    "solar_to_lunar",
    "lunar_to_solar",
    "parse_solar_date",
    "parse_lunar_date",
    "get_year_stem_branch",
    "get_month_stem_branch",
    "get_day_stem_branch",
    "get_time_stem_branch",
    "get_heavenly_stem_and_earthly_branch_date",
    "get_zodiac",
    "get_sign",
    "format_lunar_date",
    "format_chinese_date",
    "ZODIAC_NAMES",
    # Helper functions
    "get_five_elements_class",
    "get_five_elements_class_name",
    "get_time_range",
    "get_time_name",
    "calculate_nominal_age",
    "get_palace_index_by_name",
    "get_decadal_range",
    "get_decadal_palace_index",
]
